import os
import sys
import time
import datetime

import matplotlib.animation as animation
import matplotlib.pyplot as plt
import numpy as np
import scipy.stats
from skywinder_analysis.lib.tools import blosc_file
from skywinder_analysis.lib.image_processing import binning, rolling_flat_field, flat_field, flat_field_piggyback, stitching
from skywinder_analysis.lib.skywinder_analysis_app import skywinder_analysis_app


class StitchedMovieMakerApp(skywinder_analysis_app.App):
    def ani_frame_from_raw(self):
        self.create_output()
        self.logger.info('Starting animation of frames')
        imd = np.zeros(self.settings.raw_image_size)
        if self.settings.crop:
            imd = imd[self.settings.crop[0]:self.settings.crop[1],
                      self.settings.crop[2]:self.settings.crop[3]]
        frames = min([len(filenames) for filenames in self.settings.all_filenames])
        for filenames in self.settings.all_filenames:
            self.logger.info('%d filenames passed' % len(filenames))
        fig = plt.figure(frameon=False)
        ax = fig.add_subplot(111)
        ax.get_xaxis().set_visible(False)
        ax.get_yaxis().set_visible(False)
        fig.subplots_adjust(left=0, bottom=0, right=1, top=1, wspace=None, hspace=None)
        if self.settings.bin_pixels:
            img = stitching.build_stitched_image(len(self.settings.all_filenames) * [binning.bucket(imd, self.settings.bin_pixels)])
        else:
            img = stitching.build_stitched_image(len(self.settings.all_filenames) * [imd])
        if self.settings.rotate:
            self.logger.info('Shape before: %d by %d' % (img.shape[0], img.shape[1]))
            img = np.rot90(img, k=self.settings.rotate_k)
            self.logger.info('Shape after: %d by %d' % (img.shape[0], img.shape[1]))
        vsize = img.shape[0] / self.settings.dpi
        hsize = img.shape[1] / self.settings.dpi
        fig.set_size_inches([hsize, vsize])
        im = ax.imshow(img, cmap=self.settings.color_scheme, interpolation='nearest')
        xfig, yfig = fig.get_dpi() * fig.get_size_inches()
        fontsize = yfig * self.settings.font_proportion
        if len(self.settings.all_filenames) == 4:
            txt0 = ax.text(0.01 * xfig, 0.95 * yfig, 'time:', ha='left', va='top', color=self.settings.fontcolor,
                           fontsize=fontsize//2)
            txt1 = ax.text(0.26 * xfig, 0.95 * yfig, 'time:', ha='left', va='top', color=self.settings.fontcolor,
                           fontsize=fontsize//2)
            txt2 = ax.text(0.51 * xfig, 0.95 * yfig, 'time:', ha='left', va='top', color=self.settings.fontcolor,
                           fontsize=fontsize//2)
            txt3 = ax.text(0.76 * xfig, 0.95 * yfig, 'time:', ha='left', va='top', color=self.settings.fontcolor,
                           fontsize=fontsize//2)
            all_txt = [txt0, txt1, txt2, txt3]
        if len(self.settings.all_filenames) == 3:
            txt0 = ax.text(0.01 * xfig, 0.95 * yfig, 'time:', ha='left', va='top', color=self.settings.fontcolor,
                           fontsize=fontsize//2)
            txt1 = ax.text(0.34 * xfig, 0.95 * yfig, 'time:', ha='left', va='top', color=self.settings.fontcolor,
                           fontsize=fontsize//2)
            txt2 = ax.text(0.67 * xfig, 0.95 * yfig, 'time:', ha='left', va='top', color=self.settings.fontcolor,
                           fontsize=fontsize//2)
            all_txt = [txt0, txt1, txt2]
        if len(self.settings.all_filenames) == 2:
            txt0 = ax.text(0.01 * xfig, 0.95 * yfig, 'time:', ha='left', va='top', color=self.settings.fontcolor,
                           fontsize=fontsize//2)
            txt1 = ax.text(0.51 * xfig, 0.95 * yfig, 'time:', ha='left', va='top', color=self.settings.fontcolor,
                           fontsize=fontsize//2)
            all_txt = [txt0, txt1]
        if len(self.settings.all_filenames) == 1:
            txt0 = ax.text(0.01 * xfig, 0.95 * yfig, 'time:', ha='left', va='top', color=self.settings.fontcolor,
                           fontsize=fontsize//2)
            all_txt = [txt0]
        time_txt = ax.text(0.01 * xfig, 0.05 * yfig, 'time:', ha='left', va='top', color=self.settings.fontcolor,
                           fontsize=fontsize)
        start_at = time.time()
        ax = plt.gca()
        flat_fields = []
        if self.settings.saved_flat_field_paths:
            for path in self.settings.saved_flat_field_paths:
                self.logger.info('Loading saved flat field %s' % path)
                flat_field_image = np.load(path)
                flat_fields.append(flat_field_image)
            use_flat_field_image = True
            old_flat_field = True
        elif self.settings.all_flat_field_filenames:
            for filenames in self.settings.all_flat_field_filenames:
                self.logger.info('Generating flat field from %d files' % len(filenames))
                flat_field_image = flat_field.generate_flat_field_image_from_filenames(filenames)
                flat_fields.append(flat_field_image)
            use_flat_field_image = True
            old_flat_field = True
        elif self.settings.use_new_flat_field:
            self.logger.info('Using sophisticated flat field')
            use_flat_field_image = True
            flat_fields = [False, False, False, False]
            old_flat_field = False
        else:
            self.logger.info('No flat fields used')
            flat_fields = [False, False, False, False]
            use_flat_field_image = False
            old_flat_field = True

        def update_img(n):
            elapsed = time.time() - start_at
            if n:
                time_per_frame = elapsed / n
            else:
                time_per_frame = 1
            sys.stdout.flush()

            imgs = []
            for camera, txt, flat_field_image, filenames in zip(self.settings.cameras, all_txt, flat_fields,
                                                                self.settings.all_filenames):
                try:

                    if old_flat_field:
                        img, _ = blosc_file.load_blosc_image(filenames[n])
                        if use_flat_field_image:
                            img = flat_field.apply_flat_field(img, flat_field_image)
                    else:
                        if self.settings.piggyback:
                            img = flat_field_piggyback.get_final_cleaned_image(filenames[n],
                                                                               reflection_window=self.settings.new_flat_field_reflection_window)
                        else:
                            img = flat_field.get_final_cleaned_image(filenames[n],
                                                                     reflection_window=self.settings.new_flat_field_reflection_window)
                    if self.settings.crop:
                        img = img[self.settings.crop[0]:self.settings.crop[1],
                              self.settings.crop[2]:self.settings.crop[3]]

                except Exception as e:
                    self.logger.info(e)
                    return

                if self.settings.bin_pixels:
                    img = binning.bucket(img, self.settings.bin_pixels)

                imgs.append(img)

            self.logger.info(len(imgs))

            if old_flat_field:
                means = [np.mean(img[self.settings.mean_region[0]:self.settings.mean_region[1],
                                 self.settings.mean_region[2]:self.settings.mean_region[3],
                                 ]) for img in imgs]
                mean_mean = np.mean(means)
                releveled_imgs = []
                for mean, img in zip(means, imgs):
                    releveled_imgs.append((mean_mean / mean) * img)

                stitched_img = stitching.build_stitched_image(releveled_imgs)

            else:
                # No need to relevel with sophisticated flat fielding algorithm.
                stitched_img = stitching.build_stitched_image(imgs)
                if self.settings.rotate:
                    stitched_img = np.rot90(stitched_img, k=self.settings.rotate_k)

            im.set_data(stitched_img)

            filename_for_date = self.settings.all_filenames[0][n]
            unix_timestamp_s = int(filename_for_date.split('=')[-1][:10])
            dt = datetime.datetime.utcfromtimestamp(unix_timestamp_s)
            date_string = dt.strftime('%m-%d_%H:%M:%S')
            camera_string = 'Camera ' + str(camera)
            if self.settings.no_text:
                txt.set_text(' ')
                time_txt.set_text(' ')
            else:
                txt.set_text(camera_string)
                time_txt.set_text(date_string)

            if self.settings.colorscale_by_frame:
                vmin = scipy.stats.scoreatpercentile(
                    stitched_img[self.settings.level_region[0]:self.settings.level_region[1],
                    self.settings.level_region[2]:self.settings.level_region[3]],
                    0.1)
                vmax = scipy.stats.scoreatpercentile(
                    stitched_img[self.settings.level_region[0]:self.settings.level_region[1],
                    self.settings.level_region[2]:self.settings.level_region[3]],
                    99.9)
                im.set_clim(vmin, vmax)
            else:
                if not self.settings.vmin:
                    self.settings.vmin = scipy.stats.scoreatpercentile(
                        stitched_img[self.settings.level_region[0]:self.settings.level_region[1],
                        self.settings.level_region[2]:self.settings.level_region[3]],
                        0.1)
                    self.settings.vmax = scipy.stats.scoreatpercentile(
                        stitched_img[self.settings.level_region[0]:self.settings.level_region[1],
                        self.settings.level_region[2]:self.settings.level_region[3]],
                        99.9)
                im.set_clim(self.settings.vmin, self.settings.vmax)

            self.logger.info("\r%d of %d %.1f minutes elapsed, %.1f minutes remaining" % (
                n, frames, elapsed / 60,
                (frames - n) * time_per_frame / 60))
            if self.settings.individual_frames:
                fig_fn = os.path.join(self.out_path, ('frame_%d_%s.png' % (n, date_string)))
                plt.savefig(fig_fn)
                #self.logger.info('Frame saved as %s' % fig_fn)
            return im  # , txt0, txt1, txt2, txt3


        if not self.settings.individual_frames:
            if not self.settings.split_video_len:
                self.logger.info('Making continuous movie')
                ani = animation.FuncAnimation(fig, update_img, frames, interval=100)
                ani_fn = os.path.join(self.out_path, self.settings.output_name)
                ani.save(ani_fn, writer=self.settings.writer, dpi=self.settings.dpi)
                return
            else:
                full_filename_list = self.settings.all_filenames
                # Keep record of the original full set of filenames.
                num_sections = frames // self.settings.split_video_len
                self.logger.info('Number of videos: %d' % num_sections)
                for j in range(num_sections):
                    newlist = []
                    for i in range(len(full_filename_list)):
                        newlist.append(full_filename_list[i][(j*self.settings.split_video_len):((j+1)*self.settings.split_video_len)])
                    self.settings.all_filenames = newlist
                    sub_frames = len(newlist[0])
                    self.logger.info('%d sublists each of length %d' % (len(newlist), sub_frames))
                    ani = animation.FuncAnimation(fig, update_img, sub_frames, interval=100)
                    ani_fn = os.path.join(self.out_path,('movie_%d.mp4'%j))
                    ani.save(ani_fn, writer=self.settings.writer, dpi=self.settings.dpi)
                    self.logger.info("Saved movie %d" % j)
                    ani = 0
                return



        else:
            for n in range(frames):
                update_img(n)
            return


        self.logger.info('Program should never reach this point')

if __name__ == "__main__":
    app = StitchedMovieMakerApp()
    app.ani_frame_from_raw()
    app.end()



from default_settings import settings
import numpy as np
import matplotlib.animation as animation
import matplotlib.cm as cm


import os
from pmc_analysis.lib.tools import periodic_image_finder
n = 1531190700 # 2018 7 10 02 45
#n = 1531402200 # 2018 7 12 13 30
start = n - (3*60)
stop = n + (15*60)
#start = n
#stop = n + 30 
interval = 10

#settings.camera_numbers = [4,5,6,7]
#settings.all_filename_lists = {4: [], 5:[], 6:[], 7:[]}
settings.cam_nums = [5,6]
settings.all_filename_lists = {5:[], 6:[]}
for cam_num in settings.cam_nums:
    filenames = periodic_image_finder.find_periodic_images_full_fn(cam_num,
                                                                            start_timestamp=start,
                                                                            stop_timestamp=stop,
                                                                            interval=interval)

    settings.all_filename_lists[cam_num] = filenames

settings.min_x = -50
settings.max_x = 50
settings.x_increment = 1000

settings.min_y = 10
settings.max_y = 110
settings.y_increment = 1000

settings.bin_factor = 4

settings.xvalues = np.linspace(settings.min_x, settings.max_x, num=settings.x_increment)
settings.yvalues = np.linspace(settings.min_y, settings.max_y, num=settings.y_increment)

settings.m_per_pixel = 100
settings.min_physical_range = 5e3

settings.tick_spacing = 2
settings.wavelength_labels = False



import logging


class SkyWinderAnalysisAppConsoleFormatter(logging.Formatter):
    '''
    A custom formatter for SkyWinder-Analysis apps designed to
    use more verbose output for errors and debug
    messages.

    Template taken from:
        LeapAppConsoleFormatter
        http://stackoverflow.com/questions/1343227/can-pythons-logging-format-be-modified-depending-on-the-message-log-level
    '''

    def __init__(self, fmt="%(levelno)s: %(msg)s"):
        logging.Formatter.__init__(self, fmt)
        self.error_format = '%(levelname)s: %(filename)s: %(funcName)s: %(message)s'
        self.debug_format = self.error_format
        self.critical_format = self.error_format
        self.warning_format = "%(levelname)s: %(message)s"
        self.info_format = "%(message)s"

    def format(self, record):

        # Save the original format configured by the user
        # when the logger formatter was instantiated
        format_orig = self._fmt

        # Replace the original format with one customized by logging level
        if record.levelno == logging.ERROR:
            self._fmt = self.error_format
        elif record.levelno == logging.DEBUG:
            self._fmt = self.debug_format
        elif record.levelno == logging.CRITICAL:
            self._fmt = self.critical_format
        elif record.levelno == logging.WARNING:
            self._fmt = self.warning_format
        elif record.levelno == logging.INFO:
            self._fmt = self.info_format

        # Call the original formatter class to do the grunt work
        result = logging.Formatter.format(self, record)

        # Restore the original format configured by the user
        self._fmt = format_orig

        return result



import os
import sys
import time
import datetime

import netCDF4
import matplotlib.animation as animation
import matplotlib.cm as cm
import matplotlib.pyplot as plt
import numpy as np
import scipy.stats
import pandas as pd
from skywinder_analysis.lib.tools import blosc_file
from skywinder_analysis.lib.image_processing import binning, flat_field
from skywinder_analysis.lib.power_spectrum import power_spectrum_tools
from skywinder_analysis.lib.skywinder_analysis_app import skywinder_analysis_app
from scipy import interpolate
from skywinder_analysis.lib.units import coordinate_transforms


class img_container_series():
    def __init__(self, cam_num=0):
        self.cam_num = cam_num
        self.ff = False
        self.imgs = []
        self.fh = False
        self.fw = False


class StackPowerSpectrumApp(skywinder_analysis_app.App):
    def stack_power_spectra(self):
        self.create_output()
        self.logger.info('Creating container classes')
        frames = len(self.settings.all_filename_lists[self.settings.cam_nums[0]])
        ics = {}
        global_index = 0
        for cam_num in self.settings.cam_nums:
            ic = img_container_series(cam_num=cam_num)

            for fn in self.settings.all_filename_lists[cam_num]:
                if self.settings.use_new_flat_fields:
                    img = flat_field.get_final_cleaned_image(fn, self.settings.new_flat_field_window)
                    img = binning.bucket(img, (self.settings.bin_factor, self.settings.bin_factor))
                    ic.imgs.append(img)
                elif self.settings.flat_field_filenames:
                    self.logger.info('Generating flat field from %d files' % len(self.settings.flat_field_filenames[cam_num]))
                    flat_field_image = flat_field.generate_flat_field_image_from_filenames(self.settings.flat_field_filenames[cam_num])
                    img, _ = blosc_file.load_blosc_image(fn)
                    img = flat_field.apply_flat_field(img, flat_field_image)
                    img = binning.bucket(img, (self.settings.bin_factor, self.settings.bin_factor))
                    ic.imgs.append(img)
                    # Generate static flat_field_image from filenames
                else:
                    self.logger.info('Using no flat field')
                    img, _ = blosc_file.load_blosc_image(fn)
                    img = binning.bucket(img, (self.settings.bin_factor, self.settings.bin_factor))
                    ic.imgs.append(img)
            ics[cam_num] = ic

        if not self.settings.provided_grid_map:
            self.logger.info('Building interpolation functions')
            for cam_num in self.settings.cam_nums:
                df = pd.read_csv(
                    os.path.join(self.settings.pointing_directory,
                                 ('c%d_2018-07-12_2330_solution.csv' % cam_num)))
                f_h = interpolate.interp2d(df.az, df.alt, df.h,
                                           kind='cubic', bounds_error=False, fill_value=np.nan)
                f_w = interpolate.interp2d(df.az, df.alt, df.w,
                                           kind='cubic', bounds_error=False, fill_value=np.nan)
                ics[cam_num].fh = f_h
                ics[cam_num].fw = f_w

            def altaz_to_pixel(alt, az, cam_num, verbose=False):
                h = ics[cam_num].fh(az, alt)
                w = ics[cam_num].fw(az, alt)
                try:
                    h = int(np.rint(h))
                    w = int(np.rint(w))
                except ValueError as e:
                    return False
                if verbose:
                    print('H, W:', h, w)
                if h > 3231:
                    return False
                if w > 4863:
                    return False
                if h < 0:
                    return False
                if w < 0:
                    return False
                else:
                    return h, w

            def get_value(x, y, bin_factor=1, verbose=False):
                alt, az = coordinate_transforms.cart_to_altaz(x, y, verbose=verbose)
                for cam_num in self.settings.cam_nums:
                    coords = altaz_to_pixel(alt, az, cam_num, verbose=verbose)
                    if coords == False:
                        continue
                    else:
                        h, w = coords
                        return h // bin_factor, w // bin_factor, cam_num
                return -1, -1, -1

            hs = []
            ws = []
            cams = []
            for i in self.settings.xvalues:
                for j in self.settings.yvalues:
                    h, w, cam = get_value(i, j, bin_factor=self.settings.bin_factor)
                    hs.append(h)
                    ws.append(w)
                    cams.append(cam)

            map_df = pd.DataFrame(data={'h':hs, 'w':ws, 'cam':cams})
            map_df.to_csv(os.path.join(self.out_path, 'map_df.csv'))
        else:
            map_df = pd.read_csv(self.settings.provided_grid_map)

        xmin = min(self.settings.xvalues)
        xmax = max(self.settings.xvalues)
        ymin = min(self.settings.yvalues)
        ymax = max(self.settings.yvalues)

        vmins = []

        imd = np.zeros((len(self.settings.xvalues), len(self.settings.yvalues)))
        imd = np.rot90(imd, k=1)
        psd, xfreq_step, yfreq_step, xcenter, ycenter, xmaxrange, ymaxrange, xindices, yindices, xlabels, ylabels = power_spectrum_tools.get_2d_psd_and_labels(imd, m_per_pixel=self.settings.m_per_pixel,
                                                                                                                                                               min_physical_range=self.settings.min_physical_range,
                                                                                                                                                               tick_spacing=self.settings.tick_spacing,
                                                                                                                                                               wavelength_labels=self.settings.wavelength_labels)
        stacked_psd = np.zeros_like(psd)

        start_at = time.time()
        def update_psd(index):
            elapsed = time.time() - start_at
            if index:
                time_per_frame = elapsed / index
            else:
                time_per_frame = 1
            sys.stdout.flush()

            def map_image(i, j):
                #do indexing to get cam, h, and w from df
                i = int(i)
                j = int(j) 
                coords = j + (i*len(self.settings.yvalues))
                entry = map_df.iloc[coords]
                if entry.cam == -1:
                    return np.nan
                else:
                    return ics[entry.cam].imgs[index][entry.h][entry.w]

            vector_map_image = np.vectorize(map_image)
            self.logger.info('Writing frame %d' % index)
            stretched_image = np.fromfunction(vector_map_image,
                                              (len(self.settings.xvalues), len(self.settings.yvalues)))
            stretched_image = np.rot90(stretched_image, k=1)

            no_nan_image = stretched_image-np.nanmean(stretched_image)
            psd, xfreq_step, yfreq_step, xcenter, ycenter, xmaxrange, ymaxrange, xindices, yindices, xlabels, ylabels = power_spectrum_tools.get_2d_psd_and_labels(no_nan_image, m_per_pixel=self.settings.m_per_pixel,
                                                                                                                                           min_physical_range=self.settings.min_physical_range,
                                                                                                                                           tick_spacing=self.settings.tick_spacing)
            return psd

        for n in range(frames):
            new_psd = update_psd(n)
            stacked_psd += new_psd

        # Save stacked_psd to file here.
        fn = 'stacked_psd.npy'
        outpath = os.path.join(self.out_path, fn)
        with open(outpath, 'wb') as f:
            np.save(f, stacked_psd)
        return stacked_psd



if __name__ == "__main__":
    app = StackPowerSpectrumApp()
    app.stack_power_spectra()
    app.end()



from skywinder_analysis.lib.tools import blosc_file
from skywinder_analysis.lib.image_processing import binning
from skywinder_analysis.lib.image_processing import flat_field, flat_field_piggyback
import matplotlib.animation as animation
import scipy.stats
from skywinder_analysis.lib.skywinder_analysis_app import skywinder_analysis_app
import matplotlib.pyplot as plt
from matplotlib import cm
import numpy as np
from skywinder_analysis.lib.image_processing import stitching
import os
import sys
import datetime
import time
import pandas as pd
from scipy import interpolate
from skywinder_analysis.lib.units import coordinate_transforms


class img_container_series():
    def __init__(self, cam_num=0):
        self.cam_num = cam_num
        self.ff = False
        self.imgs = []
        self.fh = False
        self.fw = False


class GridMapMovieApp(skywinder_analysis_app.App):
    def project_images(self):
        self.create_output()
        self.logger.info('Creating container classes')
        frames = len(self.settings.all_filename_lists[self.settings.cam_nums[0]])
        ics = {}
        global_index = 0
        for cam_num in self.settings.cam_nums:
            ic = img_container_series(cam_num=cam_num)

            for fn in self.settings.all_filename_lists[cam_num]:
                if self.settings.use_new_flat_fields:
                    if self.settings.piggyback:
                        img = flat_field_piggyback.get_final_cleaned_image(fn, self.settings.new_flat_field_window)
                        img = binning.bucket(img, (self.settings.bin_factor, self.settings.bin_factor))
                        ic.imgs.append(img)
                    else:
                        img = flat_field.get_final_cleaned_image(fn, self.settings.new_flat_field_window)
                        img = binning.bucket(img, (self.settings.bin_factor, self.settings.bin_factor))
                        ic.imgs.append(img)
                elif self.settings.flat_field_filenames:
                    self.logger.info('Generating flat field from %d files' % len(self.settings.flat_field_filenames[cam_num]))
                    flat_field_image = flat_field.generate_flat_field_image_from_filenames(self.settings.flat_field_filenames[cam_num])
                    img, _ = blosc_file.load_blosc_image(fn)
                    img = flat_field.apply_flat_field(img, flat_field_image)
                    img = binning.bucket(img, (self.settings.bin_factor, self.settings.bin_factor))
                    ic.imgs.append(img)
                    # Generate static flat_field_image from filenames
                else:
                    self.logger.info('Using no flat field')
                    img, _ = blosc_file.load_blosc_image(fn)
                    img = binning.bucket(img, (self.settings.bin_factor, self.settings.bin_factor))
                    ic.imgs.append(img)
            ics[cam_num] = ic

        if not self.settings.provided_grid_map:
            self.logger.info('Building interpolation functions')
            for cam_num in self.settings.cam_nums:
                if self.settings.piggyback:
                    df = pd.read_csv(
                                     os.path.join(self.settings.pointing_directory,
                                                  'c99_1579217700_solution.csv'))
                else:
                    df = pd.read_csv(
                                     os.path.join(self.settings.pointing_directory,
                                     ('c%d_2018-07-12_2330_solution.csv' % cam_num)))
                f_h = interpolate.interp2d(df.az, df.alt, df.h,
                                           kind='cubic', bounds_error=False, fill_value=np.nan)
                f_w = interpolate.interp2d(df.az, df.alt, df.w,
                                           kind='cubic', bounds_error=False, fill_value=np.nan)
                ics[cam_num].fh = f_h
                ics[cam_num].fw = f_w

            def altaz_to_pixel(alt, az, cam_num, verbose=False):
                h = ics[cam_num].fh(az, alt)
                w = ics[cam_num].fw(az, alt)
                try:
                    h = int(np.rint(h))
                    w = int(np.rint(w))
                except ValueError as e:
                    return False
                if verbose:
                    print('H, W:', h, w)
                if h > 3231:
                    return False
                if w > 4863:
                    return False
                if h < 0:
                    return False
                if w < 0:
                    return False
                else:
                    return h, w

            def get_value(x, y, bin_factor=1, verbose=False):
                alt, az = coordinate_transforms.cart_to_altaz(x, y, centerline=self.settings.centerline, verbose=verbose)
                for cam_num in self.settings.cam_nums:
                    coords = altaz_to_pixel(alt, az, cam_num, verbose=verbose)
                    if coords == False:
                        continue
                    else:
                        h, w = coords
                        return h // bin_factor, w // bin_factor, cam_num
                return -1, -1, -1

            hs = []
            ws = []
            cams = []
            for i in self.settings.xvalues:
                for j in self.settings.yvalues:
                    h, w, cam = get_value(i, j, bin_factor=self.settings.bin_factor)
                    hs.append(h)
                    ws.append(w)
                    cams.append(cam)

            map_df = pd.DataFrame(data={'h':hs, 'w':ws, 'cam':cams})
            map_df.to_csv(os.path.join(self.out_path, 'map_df.csv'))
        else:
            map_df = pd.read_csv(self.settings.provided_grid_map)

        xmin = min(self.settings.xvalues)
        xmax = max(self.settings.xvalues)
        ymin = min(self.settings.yvalues)
        ymax = max(self.settings.yvalues)

        vmins = []
        #vmaxs = []
        #for cam_num in self.settings.cam_nums:
        #    vmin = scipy.stats.scoreatpercentile(
        #        ics[cam_num].imgs[0][self.settings.level_region[0]:self.settings.level_region[1],
        #        self.settings.level_region[2]:self.settings.level_region[3]]
        #        , .01)
        #    vmax = scipy.stats.scoreatpercentile(
        #        ics[cam_num].imgs[0][self.settings.level_region[0]:self.settings.level_region[1],
        #        self.settings.level_region[2]:self.settings.level_region[3]]
        #        , 99.9)
        #    vmins.append(vmin)
        #    vmaxs.append(vmax)

        #vmin = min(vmins)
        #vmax = max(vmaxs)


        imd = np.zeros((len(self.settings.xvalues), len(self.settings.yvalues)))
        imd = np.rot90(imd, k=1)

        fig = plt.figure(frameon=False)
        ax = fig.add_subplot(111)
        #im = ax.imshow(imd, vmin=vmin, vmax=vmax, cmap=self.settings.color_scheme,
        #               origin='lower', extent=[xmin, xmax, ymax, ymin])

        im = ax.imshow(imd, cmap=self.settings.color_scheme,
                       origin='lower', extent=[xmin, xmax, ymax, ymin])

        if self.settings.wireframes:
            def altaz_to_cart(alt, az, h=(83-38)):
                theta = 90-alt # Zero degrees is straight up
                phi = az - 100 # 100 degrees is roughly center of these particular solutions.
                l = h*np.tan(np.deg2rad(theta))
                y = l * np.cos(np.deg2rad(phi))
                x = l * np.sin(np.deg2rad(phi))
                return x, y
            # Narrow field wireframes
            for cam_num in ['1','2','3']:
                df = pd.read_csv(self.settings.narrowfield_wireframe_solution_directories[narrow_cam_num])
                for sub_df in [df.loc[df.h == 0],
                       df.loc[df.h == 3232],
                       df.loc[df.w == 4864],
                       df.loc[df.w == 0]]:
                    x_km = []
                    y_km = []
                    for az, alt in zip(sub_df.az, sub_df.alt):
                        x, y = altaz_to_cart(alt, az)
                        x_km.append(x)
                        y_km.append(y)
                    ax.plot(x_km, y_km, '-', color='k')

        #ax.set_aspect('equal')
        #ax.set_axisbelow(True)
        if self.settings.grid:
            #ax.set_xlabel('x (km)')
            #ax.set_ylabel('y (km)')
            ax.xaxis.set_major_locator(plt.MultipleLocator(self.settings.grid_spacing))
            ax.yaxis.set_major_locator(plt.MultipleLocator(self.settings.grid_spacing))
            ax.grid(True)
        else:
            ax.grid(False)
            ax.get_xaxis().set_visible(False)
            ax.get_yaxis().set_visible(False)
        fig.subplots_adjust(left=0, bottom=0, right=1, top=1, wspace=None, hspace=None)
        vsize = imd.shape[0] / self.settings.dpi
        hsize = imd.shape[1] / self.settings.dpi
        fig.set_size_inches([hsize, vsize])


        xfig, yfig = fig.get_dpi() * fig.get_size_inches()
        fontsize = yfig / 24
        if self.settings.timestamp_in_image:
            time_txt = ax.text(0.05, .95, 'time:', ha='left', va='top', color=self.settings.text_color,
                               fontsize=fontsize, transform=ax.transAxes)
        else:
            time_txt = False
        start_at = time.time()
        def update_img(index):
            elapsed = time.time() - start_at
            if index:
                time_per_frame = elapsed / index
            else:
                time_per_frame = 1
            sys.stdout.flush()

            def map_image(i, j):
                #do indexing to get cam, h, and w from df
                i = int(i)
                j = int(j) 
                coords = j + (i*len(self.settings.yvalues))
                entry = map_df.iloc[coords]
                if entry.cam == -1:
                    return np.nan
                else:
                    return ics[entry.cam].imgs[index][entry.h][entry.w]

            vector_map_image = np.vectorize(map_image)
            self.logger.info('Writing frame %d' % index)
            stretched_image = np.fromfunction(vector_map_image,
                                              (len(self.settings.xvalues), len(self.settings.yvalues)))
            stretched_image = np.rot90(stretched_image, k=1)

            filename_for_date = self.settings.all_filename_lists[self.settings.cam_nums[0]][index]
            unix_timestamp_s = int(filename_for_date.split('=')[-1][:10])
            dt = datetime.datetime.utcfromtimestamp(unix_timestamp_s)
            date_string = dt.strftime('%m-%d_%H:%M:%S')

            if self.settings.timestamp_in_image:
                time_txt.set_text(date_string)

            if True:
                vmin = np.nanpercentile(stretched_image, .01)
                vmax = np.nanpercentile(stretched_image, 99.99)
                print(vmin, vmax)
            im.set_data(stretched_image)
            im.set_clim(vmin=vmin, vmax=vmax)

            if self.settings.individual_frames:
                fig_fn = os.path.join(self.out_path, ('frame_%d_%s.png' % (n, date_string)))
                plt.savefig(fig_fn)
                self.logger.info('Frame saved as %s' % fig_fn)
            self.logger.info("\r%d of %d %.1f minutes elapsed, %.1f minutes remaining" % (index, frames, elapsed/60, (frames-index) * time_per_frame / 60))
            return im, time_txt

        if not self.settings.individual_frames:
            ani = animation.FuncAnimation(fig, update_img, frames, interval=500)
            ani_fn = os.path.join(self.out_path, 'movie.mp4')
            ani.save(ani_fn, writer=self.settings.writer, dpi=self.settings.dpi)
            return ani
        else:
            for n in range(frames):
                update_img(n)
            return True


        return ani

if __name__ == "__main__":
    app = GridMapMovieApp()
    app.project_images()



from __future__ import division

import numpy as np
from scipy.interpolate import interp1d
import os
import datetime as dt

from skywinder_analysis.lib.tools import blosc_file
from skywinder_analysis.lib.image_processing import sky_brightness_model, dark_images


def get_final_cleaned_image(fn, reflection_window=600):
    #Reflection window should be given in seconds. The actual window used is the value given rounded down to the nearest 10 minutes (or rounded up to 10 minutes if less than 10 minutes)
    #Returns final cleaned image in units of counts/second, with some arbitrary offset.
    camera_number = int(fn.split('/')[4][1])
    timestamp = int(fn.split('=')[-1][:10])
    
    calibrated_image = get_foreground_subtracted_image(camera_number, fn)
    reflection_image = get_composite_flat(camera_number, timestamp, reflection_window)

    return 1000000.0*(calibrated_image - reflection_image)


def get_final_cleaned_image_aurora(fn, reflection_window=60):
    #Reflection window should be given in seconds. The actual window used is the value given rounded down to the nearest 10 minutes (or rounded up to 10 minutes if less than 10 minutes)
    #Returns final cleaned image in units of counts/second, with some arbitrary offset.
    camera_number = int(fn.split('/')[4][1])
    timestamp = int(fn.split('=')[-1][:10])
    
    calibrated_image = get_foreground_subtracted_image(camera_number, fn)
    reflection_image = get_composite_flat_aurora(camera_number, timestamp, reflection_window)

    return 1000000.0*(calibrated_image - reflection_image)

def get_calibrated_image(camera_number, fn):
    timestamp = int(fn.split('=')[-1][:10])
    
    img, chunk = blosc_file.load_blosc_image(fn)
    exposure = float(chunk[0][6])
    
    flat = get_flat_field_image_pre_flight(camera_number)
    dark = dark_images.get_dark_image(camera_number, exposure)
    
    calibrated_image = ((img.astype('float') - dark.astype('float'))/flat.astype('float'))*np.mean(flat)
    return calibrated_image

def get_foreground_subtracted_image(camera_number, fn):
    timestamp = int(fn.split('=')[-1][:10])
    
    img, chunk = blosc_file.load_blosc_image(fn)
    exposure = float(chunk[0][6])
    
    flat = get_flat_field_image_pre_flight(camera_number)
    sky = sky_brightness_model.simulate_sky_in_flight(camera_number, timestamp)
    dark = dark_images.get_dark_image(camera_number, exposure)
    
    flat_image = ((img.astype('float') - dark.astype('float'))/flat.astype('float'))*np.mean(flat)
    foreground_subtracted_image = ((flat_image.astype('float') - sky.astype('float')*np.mean(flat_image)/np.mean(sky))/exposure)
    return foreground_subtracted_image


def get_composite_flat(camera_number, timestamp, reflection_window = 600):
    N = np.max([reflection_window // 600, 1]) 
    direc = '/data/mounted_filesystems/nas2/resources/foreground_subtracted_composites_new/c{0}/'.format(camera_number)
    fns = sorted(os.listdir(direc))
    time_list = []
    for fn in fns:
        ts = dt.datetime.strptime(fn[30:47], '%m_%d_%Y--%H:%M').timestamp() - 14100
        time_list.append(ts)

    times = np.asarray(time_list)
    index = np.where(times <= timestamp)[0][-1]
    if N % 2 == 0:
        if timestamp - times[index] > 300:
            index += 1
            
            
    imd = np.zeros((3232, 4864, 2))
    img = np.zeros((3232, 4864))
    
    for i in range(N - 1):
        temp, _ = blosc_file.load_blosc_image(os.path.join(direc, fns[index - N // 2 + 1 + i]))
        img += temp.astype('float')/float(N)
    
    imd[:,:,0], _ = blosc_file.load_blosc_image(os.path.join(direc, fns[index - N // 2]))
        
    x = np.array([times[index], times[index + 1]])
    if N % 2 == 0:
        x -= 300
        imd[:,:,1], _ = blosc_file.load_blosc_image(os.path.join(direc, fns[index + N // 2]))
    else:
        imd[:,:,1], _ = blosc_file.load_blosc_image(os.path.join(direc, fns[index + N // 2 + 1]))
    
    img_interp = interp1d(x, imd)
    
    img += img_interp(timestamp)/float(N)
    
    img0 = img.astype('float')/20000.0
    
    img1 = img0 - 2*np.min(img0)

    return img1

def get_composite_flat_aurora(camera_number, timestamp, reflection_window = 60):
    N = np.max([reflection_window // 60, 1]) 
    direc = '/data/mounted_filesystems/nas2/resources/foreground_subtracted_composites_1min/c{0}/'.format(camera_number)
    fns = sorted(os.listdir(direc))

    times = []
    for fn in fns:
        day = day = int(fn[33:35])
        time = 1530403200 + 86400*(day-1) + int(fn[-5:-3])*3600 + int(fn[-2:])*60 - 30
        times.append(time)
    times = np.asarray(times)

    index = np.where(times <= timestamp)[0][-1]
    if N % 2 == 0:
        if timestamp - times[index] > 30:
            index += 1
            
            
    imd = np.zeros((3232, 4864, 2))
    img = np.zeros((3232, 4864))
    
    for i in range(N - 1):
        temp, _ = blosc_file.load_blosc_image(os.path.join(direc, fns[index - N // 2 + 1 + i]))
        img += temp.astype('float')/float(N)
    
    imd[:,:,0], _ = blosc_file.load_blosc_image(os.path.join(direc, fns[index - N // 2]))
        
    x = np.array([times[index], times[index + 1]])
    if N % 2 == 0:
        x -= 30
        imd[:,:,1], _ = blosc_file.load_blosc_image(os.path.join(direc, fns[index + N // 2]))
    else:
        imd[:,:,1], _ = blosc_file.load_blosc_image(os.path.join(direc, fns[index + N // 2 + 1]))
    
    img_interp = interp1d(x, imd)
    
    img += img_interp(timestamp)/float(N)
    
    img0 = img.astype('float')/20000.0
    
    img1 = img0 - 2*np.min(img0)

    return img1

def get_flat_field_image_pre_flight(camera_number):
    flat, _ = blosc_file.load_blosc_image('/data/mounted_filesystems/nas2/resources/flat_fields_pre-flight/camera{0}_flat_field'.format(camera_number))
    return flat

def get_uncorrected_flat_field_image_pre_flight(camera_number):
    flat, _ = blosc_file.load_blosc_image('/home/christopher/SkyWinder-Analysis/skywinder_analysis/resources/flat_fields/camera{0}_uncorrected_flat_field'.format(camera_number))
    return flat

def generate_flat_field_image_from_filenames(fns):
    images = []
    for fn in fns:
        try:
            image, _ = blosc_file.load_blosc_image(fn)
            images.append(image)
        except Exception as e:
            print(e)
            print(fn)
    return generate_flat_field_image_from_raw(images)


def generate_flat_field_image_from_raw(images):
    image_array = np.array(images)
    return np.average(image_array, axis=0)


def generate_flat_field_running_average(fns):
    ff_img = np.zeros((3232, 4864))
    images_added = 0
    for fn in fns:
        try:
            image, _ = blosc_file.load_blosc_image(fn)
            ff_img = (images_added * ff_img) / (images_added + 1) + image / (images_added + 1)
            images_added += 1
        except Exception as e:
            print(e)
            print(fn)
    return ff_img


def apply_flat_field(image, flat_field_image):
    return image * np.mean(flat_field_image) / flat_field_image


def apply_flat_field_no_relevel(image, flat_field_image):
    return image / flat_field_image


def apply_flat_field_subtract(image, flat_field_image):
    return (image + np.mean(flat_field_image)) - flat_field_image



from default_settings import settings
import matplotlib.animation as animation
import os
from pmc_analysis.lib.tools import periodic_image_finder
import matplotlib.cm as cm
import datetime
import calendar

n = 1531298580 # 2018-07-11_04430
start = n
stop = start + 2
interval = 2 # One frame every 20 minutes

c3_filenames = periodic_image_finder.find_periodic_images_full_fn(3, start, stop, interval)
settings.all_filenames = [c3_filenames]
settings.cameras = [3]

settings.use_new_flat_field = True
settings.new_flat_field_reflection_window = 600


settings.bin_pixels = (4,4)

settings.level_region = [4864 // (4 * settings.bin_pixels[0]),
                         (4864) // (settings.bin_pixels[0]),#(3 * 4864) // (4 * settings.bin_pixels[0]),
                         0 // settings.bin_pixels[1],
                         (len(settings.all_filenames) * 3232) // settings.bin_pixels[1]]
# Level full stitched by central part of all cameras



settings.color_scheme = cm.inferno
#settings.fontcolor = 'white'
settings.crop = [3232//4,3232,4864//4,5*4864//8]
settings.colorscale_by_frame = True
settings.vmin = False
settings.vmax = False

settings.individual_frames = True
#settings.split_video_len = (20*60 // interval)
settings.font_proportion = 1./16.
settings.no_text = True



import numpy as np
import matplotlib.pyplot as plt
import matplotlib.animation as animation
import scipy.stats
import matplotlib.cm as cm
import time
import binning
import flat_field
import blosc_file
import sys


def ani_frame_from_intermediate_data_products(fns, output_name, bin_pixels=(1, 1), section=None):
    imd = np.zeros((3232, 4864))
    if section:
        imd = imd[section[0]:section[1], section[2]:section[3]]
    dpi = 100
    vmin = 0
    vmax = 0

    frames = len(fns)
    fig = plt.figure()
    ax = fig.add_subplot(111)
    ax.get_xaxis().set_visible(False)
    ax.get_yaxis().set_visible(False)

    if bin_pixels:
        img = binning.bucket(imd, bin_pixels)
    else:
        img = imd

    im = ax.imshow(img, cmap=cm.inferno, interpolation='nearest')

    fig.set_size_inches([12, 8])
    txt = ax.text(0.1, 0.9, 'time:', ha='left', va='top', color='yellow')
    plt.tight_layout()
    start_at = time.time()
    ax = plt.gca()

    def update_img(n):
        elapsed = time.time() - start_at
        if n:
            time_per_frame = elapsed / n
        else:
            time_per_frame = 1
        sys.stdout.flush()
        try:
            img = np.load(fns[n])
        except Exception:
            return
        if section:
            img = img[section[0]:section[1], section[2]:section[3]]
        if bin_pixels:
            img = binning.bucket(img, bin_pixels)
        im.set_data(img)

        # if vmin == 0:
        vmin = scipy.stats.scoreatpercentile(img, 0.1)
        vmax = scipy.stats.scoreatpercentile(img, 99.9)
        im.set_clim(vmin, vmax)

        try:
            time_string = fns[n].split('/')[-1].split('_')[1]
            time_string = time_string[0:2] + ':' + time_string[2:4] + ':' + time_string[4:]
            txt.set_text(time_string)
        except Exception:
            pass
        print("\r%d of %d %.1f minutes elapsed, %.1f minutes remaining" % (n, len(fns), elapsed / 60,
                                                                           (len(fns) - n) * time_per_frame / 60))

        return im, txt

    ani = animation.FuncAnimation(fig, update_img, frames, interval=100)
    ani.save(output_name, dpi=dpi)
    return ani


def ani_frame_from_raw(fns, output_name, saved_flat_field=None, ff_fns=None, bin_pixels=(1, 1), section=None):
    imd = np.zeros((3232, 4864))
    if section:
        imd = imd[section[0]:section[1], section[2]:section[3]]
    dpi = 100
    vmin = 0
    vmax = 0

    frames = len(fns)
    fig = plt.figure()
    ax = fig.add_subplot(111)
    ax.get_xaxis().set_visible(False)
    ax.get_yaxis().set_visible(False)

    if bin_pixels:
        img = binning.bucket(imd, bin_pixels)
    else:
        img = imd

    im = ax.imshow(img, cmap=cm.inferno, interpolation='nearest')

    fig.set_size_inches([12, 8])
    txt = ax.text(0.1, 0.9, 'time:', ha='left', va='top', color='yellow')
    plt.tight_layout()
    start_at = time.time()
    ax = plt.gca()

    if saved_flat_field:
        print('Loading saved flat field %s' % saved_flat_field)
        flat_field_image = np.load(saved_flat_field)
        use_flat_field_image = True
    elif ff_fns:
        print('Generating flat field')
        flat_field_image = flat_field.generate_flat_field_image_from_files(ff_fns)
        use_flat_field_image = True
        # Generate flat_field_image from filenames
    else:
        print = ('No flat field used')
        use_flat_field_image = False

    def update_img(n):
        elapsed = time.time() - start_at
        if n:
            time_per_frame = elapsed / n
        else:
            time_per_frame = 1
        sys.stdout.flush()
        try:
            img, _ = blosc_file.load_blosc_image(fns[n])
            if use_flat_field_image:
                img = flat_field.apply_flat_field(img, flat_field_image)
        except Exception as e:
            print(e)
            return
        if section:
            img = img[section[0]:section[1], section[2]:section[3]]
        if bin_pixels:
            img = binning.bucket(img, bin_pixels)
        im.set_data(img)

        # if vmin == 0:
        vmin = scipy.stats.scoreatpercentile(img, 0.1)
        vmax = scipy.stats.scoreatpercentile(img, 99.9)
        im.set_clim(vmin, vmax)

        try:
            time_string = fns[n].split('/')[-1].split('_')[1]
            time_string = time_string[0:2] + ':' + time_string[2:4] + ':' + time_string[4:]
            txt.set_text(time_string)
        except Exception:
            pass
        print("\r%d of %d %.1f minutes elapsed, %.1f minutes remaining" % (n, len(fns), elapsed / 60,
                                                                           (len(fns) - n) * time_per_frame / 60))

        return im, txt

    ani = animation.FuncAnimation(fig, update_img, frames, interval=100)
    ani.save(output_name, dpi=dpi)
    return ani


if __name__ == "__main__":
    import glob

    fns = glob.glob(sys.argv[1])
    # Include quotes around the path
    output_name = sys.argv[2]
    ff_fns = fns[::10]
    ani_frame_from_raw(fns, output_name, saved_flat_field=None, ff_fns=ff_fns, bin_pixels=(4, 4), section=None)



import numpy as np

from skywinder_analysis.lib.tools import  relative_camera_pointing
from skywinder_analysis.lib.tools import GPS_pointing_and_sun as GPS
from scipy.interpolate import  RectBivariateSpline as RBS

def simulate_sky_in_flight(camera_number, img_time):
    sun_az, sun_alt = GPS.get_sun_az_alt(img_time)
    camera_az_offset = GPS.get_pointing_rotator(img_time) - sun_az + np.pi
    
    sky_brightness = np.zeros((5, 5))
    az, alt = relative_camera_pointing.get_camera_pointing_array(camera_number)
    az += camera_az_offset

    for i in range(5):
        for j in range(5):
            sky_brightness[i, j] = 1./np.sin(alt[i, j])*(1 + 
                  (np.cos(az[i, j])*np.cos(sun_alt)*np.cos(alt[i, j]) + np.sin(alt[i, j])*np.sin(sun_alt))**2)
    
    x_old = np.arange(0, 3233, 808)
    y_old = np.arange(0, 4865, 1216)

    x_new = np.arange(0,3232,1)
    y_new = np.arange(0,4864,1)
    
    f_sky = RBS(x_old, y_old, sky_brightness)
    
    full_sky_brightness = f_sky(x_new, y_new)
    return full_sky_brightness



import numpy as np


def cart_to_altaz(x, y, h=(83 - 38), centerline=100, verbose=False):
    # Cartesian coordinates in km on the pmc plane to alt/az
    l = np.sqrt(x ** 2 + y ** 2)
    theta = np.arctan(l / h)
    phi = np.arcsin(x / l)
    alt = 90 - np.rad2deg(theta)
    az = centerline + np.rad2deg(phi)
    if verbose:
        print('Cartesian x, y:', x, y)
        print('Alt az:', alt, az)
    return alt, az


def altaz_to_cart(alt, az, phi_offset=100, h=(83 - 38)):
    # Alt az to cartesian coordinates in km on the pmc plane.
    # Phi offset is the anti-sun angle if x=0 corresponding to anti sun direction is desired.
    theta = 90 - alt  # Zero degrees is straight up
    phi = az - phi_offset  # 100 degrees is roughly center of these particular solutions.
    l = h * np.tan(np.deg2rad(theta))
    y = l * np.cos(np.deg2rad(phi))
    x = l * np.sin(np.deg2rad(phi))
    return x, y



import os
import sys
import time
import datetime

import matplotlib.animation as animation
import matplotlib.cm as cm
import matplotlib.pyplot as plt
import numpy as np
import scipy.stats
from skywinder_analysis.lib.tools import blosc_file
from skywinder_analysis.lib.image_processing import binning, rolling_flat_field, flat_field
from skywinder_analysis.lib.skywinder_analysis_app import skywinder_analysis_app


class RollingMovieMakerApp(skywinder_analysis_app.App):
    def ani_frame_from_raw(self):
        self.create_output()
        self.logger.info('Starting animation of frames')
        imd = np.zeros(self.settings.raw_image_size)
        if self.settings.section:
            imd = imd[self.settings.section[0]:self.settings.section[1],
                  self.settings.section[2]:self.settings.section[3]]

        frames = len(self.settings.filenames)
        self.logger.info('%d filenames passed' % frames)
        fig = plt.figure(frameon=False)
        ax = fig.add_subplot(111)
        ax.get_xaxis().set_visible(False)
        ax.get_yaxis().set_visible(False)
        fig.subplots_adjust(left=0, bottom=0, right=1, top=1, wspace=None, hspace=None)

        if self.settings.bin_pixels:
            img = binning.bucket(imd, self.settings.bin_pixels)
        else:
            img = imd
        vsize = img.shape[0] / self.settings.dpi
        hsize = img.shape[1] / self.settings.dpi
        fig.set_size_inches([hsize, vsize])

        im = ax.imshow(img, cmap=cm.inferno, interpolation='nearest')

        txt = ax.text(0.1, 0.9, 'time:', ha='left', va='top', color='yellow')
        # plt.tight_layout()
        start_at = time.time()
        ax = plt.gca()

        rolling_ff = rolling_flat_field.RollingFlatField(size=self.settings.rolling_flat_field_size)
        for i in range(self.settings.rolling_flat_field_size // 2):
            rolling_ff.roll(self.settings.filenames[i])

        # Populate rolling flat field with first size/2 images.

        def update_img(n):
            elapsed = time.time() - start_at
            if n:
                time_per_frame = elapsed / n
            else:
                time_per_frame = 1
            sys.stdout.flush()
            try:
                if n > (len(self.settings.filenames) - (self.settings.rolling_flat_field_size / 2)):
                    pass
                else:
                    m = n + (self.settings.rolling_flat_field_size // 2)
                    rolling_ff.roll(self.settings.filenames[m])
                    self.logger.info('Rolling to index %d for ff on image %d' % (m, n))
                flat_field_image = rolling_ff.generate_flat_field()
                img, _ = blosc_file.load_blosc_image(self.settings.filenames[n])
                img = flat_field.apply_flat_field(img, flat_field_image)
            except Exception as e:
                self.logger.info(e)
                return
            if self.settings.section:
                img = img[self.settings.section[0]:self.settings.section[1],
                      self.settings.section[2]:self.settings.section[3]]
            if self.settings.bin_pixels:
                img = binning.bucket(img, self.settings.bin_pixels)
            im.set_data(img)

            # if vmin == 0:
            vmin = scipy.stats.scoreatpercentile(img[self.settings.level_region[0]:self.settings.level_region[1],
                                                 self.settings.level_region[2]:self.settings.level_region[3]], 0.1)
            vmax = scipy.stats.scoreatpercentile(img[self.settings.level_region[0]:self.settings.level_region[1],
                                                 self.settings.level_region[2]:self.settings.level_region[3]], 99.9)
            im.set_clim(vmin, vmax)

            unix_timestamp_s = int(self.settings.filenames[n].split('=')[-1][:10])
            dt = datetime.datetime.utcfromtimestamp(unix_timestamp_s)
            date_string = dt.strftime('%m-%d %H:%M:%S')
            txt.set_text(date_string)
            self.logger.info("\r%d of %d %.1f minutes elapsed, %.1f minutes remaining" % (
                n, len(self.settings.filenames), elapsed / 60,
                (len(self.settings.filenames) - n) * time_per_frame / 60))

            return im, txt

        ani = animation.FuncAnimation(fig, update_img, frames, interval=100)

        ani_fn = os.path.join(self.out_path, self.settings.output_name)
        ani.save(ani_fn, writer=self.settings.writer, dpi=self.settings.dpi)
        return ani


if __name__ == "__main__":
    app = RollingMovieMakerApp()
    app.ani_frame_from_raw()
    app.end()



from default_settings import settings
import os
from pmc_analysis.lib.tools import periodic_image_finder
import matplotlib.animation as animation

n = 1531038600 # 2018-07-08_0830
start = n + (3*60*60)
end = start + (16*60*60)
interval = 60

c4_filenames = periodic_image_finder.find_periodic_images(4, start, end, interval)
c5_filenames = periodic_image_finder.find_periodic_images(5, start, end, interval)
c6_filenames = periodic_image_finder.find_periodic_images(6, start, end, interval)
c7_filenames = periodic_image_finder.find_periodic_images(7, start, end, interval)
nas1 = [c5_filenames, c6_filenames, c7_filenames]
nas2 = [c4_filenames]
settings.all_filenames = []
for filenames in nas2:
    joined_filenames = []
    for fn in filenames:
        joined_filenames.append(os.path.join(('/data/mounted_filesystems/nas1/c%d/'%4), fn))
    settings.all_filenames.append(joined_filenames)
for camera_number, filenames in zip([5,6,7],nas1):
    joined_filenames = []
    for fn in filenames:
        joined_filenames.append(os.path.join(('/data/mounted_filesystems/nas2/c%d/'%camera_number), fn))
    settings.all_filenames.append(joined_filenames)

settings.rolling_flat_field_size = 10
settings.writer = animation.writers['ffmpeg'](fps=20, codec='libx264', bitrate=2 ** 20)
settings.bin_pixels = (16,16)
settings.level_region = [4864 // (4 * settings.bin_pixels[0]),
                         (3 * 4864) // (4 * settings.bin_pixels[0]),
                         (3232 * 2) // settings.bin_pixels[1],
                         (3232 * 3) // settings.bin_pixels[1]]
# Level full stitched by central part of camera 6

settings.mean_region = [0 // settings.bin_pixels[0],
                        3232 // settings.bin_pixels[0],
                        4864 // (4 * settings.bin_pixels[1]),
                        (3 * 4864) // (4 * settings.bin_pixels[1])]
# Compare mean of images using central 50%



import datetime
import imp
import inspect
import logging
import os
import sys
import time
import skywinder_analysis
import pep8
import multiprocessing
import shutil
import pprint
import matplotlib.pyplot as plt
from skywinder_analysis.lib.units import times
from skywinder_analysis.lib.tools import letter_cases
from skywinder_analysis.lib.tools.skywinder_analysis_termcolor import colored
from skywinder_analysis.lib.skywinder_analysis_app.skywinder_analysis_app_console_formatter import SkyWinderAnalysisAppConsoleFormatter


class App(object):
    '''
    Base app object for SkyWinder-Analysis apps.
    - Defines output directory asd self.out_path
    - Will create output directory with self.create_output
    - Checks code for PEP8 compliance.
    - Tracks running time
    - Defines a logger with standard default operation
    '''

    def __init__(self, settings=None, out_path=None, check_pep8=False):
        self.__start_time = time.time()
        self.__run_time_string = datetime.datetime.fromtimestamp(
            int(self.__start_time)).strftime("%Y-%m-%d--%H-%M-%S")
        self.__camel_case_name = self.__class__.__name__
        self.__name = letter_cases.from_camel_to_lower(self.__camel_case_name)
        if self.__name.endswith("_app"):
            self.__name = self.__name[:-4]
        self.pmca_path = os.path.realpath(skywinder_analysis.__path__[0])
        self.resources_path = os.path.join(self.pmca_path, 'resources')
        self.ldb_data_path = os.path.join(self.pmca_path, 'ldb_data')
        self.long_term_output_path = os.path.join(self.pmca_path, 'long_term_output')
        self.app_dir = os.path.dirname(os.path.realpath(inspect.getfile(self.__class__)))
        self.logger, self.console_logger = self._create_console_logger_and_handler(self.app_dir, self.pmca_path)
        if out_path is not None:
            self.out_path = out_path
        else:
            self.out_path = self.__define_out_path()
        self.out_admin_path = os.path.join(self.out_path, "admin")
        if settings is not None:
            self.settings = settings
        else:
            self.custom_settings_path = os.path.join(self.app_dir, 'custom_settings.py')
            self.default_settings_path = os.path.join(self.app_dir, 'default_settings.py')
            if not os.path.exists(self.default_settings_path):
                error_message = ''.join(["\nPlease update your App's folder to conform the SkyWinder-Analysis convention\n",
                                         "You need to create/commit default_settings.py to the repo\n",
                                         "default_settings.py must contain the minimum settings required\n",
                                         "Create a git ignored file called custom_settings.py\n",
                                         "You can override or add addtional settings in custom_settings.py"])
                raise (IOError(error_message))
            if os.path.exists(self.default_settings_path) and not os.path.exists(self.custom_settings_path):
                error_message = ''.join(["\nPlease update your App's folder to conform the SkyWinder-Analysios convention\n",
                                         "Create a git ignored file called custom_settings.py\n",
                                         "In this file put at least this line:\n"
                                         "from default_settings import settings\n",
                                         "Then you can override or add to this settings object"])
                raise (IOError(error_message))
            self._load_settings()
        self.logger.info(colored("==== running %s (%s) ====" % (self.__name, self.__run_time_string), "cyan"))
        if check_pep8:
            self.check_pep8()

    def _load_settings(self):
        if os.path.isfile(self.custom_settings_path):
            settings_module = imp.load_source('settings', self.custom_settings_path)
            if hasattr(settings_module, "settings"):
                self.settings = settings_module.settings

    def _create_console_logger_and_handler(self, app_dir, pmca_dir):
        '''
        Creates a console hander with the format defined in
        SkyWinderAnalysisAppConsoleFormatter. Creates a logger with this handler
        and returns both
        '''
        # logger name is made up of alg, process name and time to ensure uniqueness
        alg_path = os.path.relpath(app_dir, pmca_dir)
        alg_path = os.path.normpath(alg_path)  # converts to standard unix path
        proc_name = multiprocessing.current_process().name
        name = "%s.%s.%s" % (alg_path.replace('/', '.'), proc_name, str(time.time()).replace('.', ''))
        logger = logging.getLogger(name)
        console = logging.StreamHandler(sys.stdout)
        formatter = SkyWinderAnalysisAppConsoleFormatter()  # use custom formatter for console
        console.setFormatter(formatter)
        try:
            console.set_name('console')
        except AttributeError:
            pass
        if not console in logger.handlers:
            logger.addHandler(console)
        logger.setLevel(logging.DEBUG)  # Control levels via handlers
        return logger, console

    def _create_file_logger(self, log_path, level=logging.DEBUG):
        file_logger = logging.FileHandler(log_path)
        formatter = logging.Formatter('%(asctime)s | %(levelname)s | %(filename)s | %(funcName)s | %(message)s')
        file_logger.setFormatter(formatter)
        file_logger.setLevel(level)
        try:
            file_logger.set_name('file')
        except AttributeError:
            pass
        return file_logger

    def __define_out_path(self):
        '''
            Constructs the path for output by combining the start_time with
            the name of the child class.
        '''
        return os.path.join(self.pmca_path, "output",
                            self.__run_time_string + "_" + self.__name)

    def check_pep8(self, quiet=True):
        guide = pep8.StyleGuide(quiet=quiet, max_line_length=120)
        report = guide.check_files(".")
        if report.get_count() > 0:
            self.logger.info(colored("==== (files do not meet style guidelines) ====", "cyan"))

    def create_output(self, write_git_diff=True):
        if not os.path.isdir(self.out_path):
            os.mkdir(self.out_path)
            self.logger.info(colored("==== created output for %s (%s) ====" % (self.__name, self.__run_time_string),
                                     "cyan"))
        if not os.path.isdir(self.out_admin_path):
            os.mkdir(self.out_admin_path)
        if hasattr(self, "custom_settings_path"):
            if os.path.exists(self.custom_settings_path):
                if not os.path.isfile(os.path.join(self.out_admin_path, "custom_settings.py")):
                    shutil.copy(self.custom_settings_path, os.path.join(self.out_admin_path, "custom_settings.py"))
        if hasattr(self, "default_settings_path"):
            if os.path.exists(self.default_settings_path):
                if not os.path.isfile(os.path.join(self.out_admin_path, "default_settings.py")):
                    shutil.copy(self.default_settings_path, os.path.join(self.out_admin_path, "default_settings.py"))
        if hasattr(self, "settings"):
            if not os.path.isfile(os.path.join(self.out_admin_path, 'settings_used.txt')):
                with open(os.path.join(self.out_admin_path, 'settings_used.txt'), 'w') as fh:
                    pprinter = pprint.PrettyPrinter(width=120, stream=fh)
                    pprinter.pprint(vars(self.settings))

        # Create log file and the handler for logging
        proc_name = multiprocessing.current_process().name
        log_path = os.path.join(self.out_admin_path, proc_name) + '.txt'
        if not hasattr(self, "file_logger"):
            self.file_logger = self._create_file_logger(log_path)
        if not self.file_logger in self.logger.handlers:
            self.logger.addHandler(self.file_logger)

            # svn state
            # if write_svn_diff:
            #     svn_tool = svn.SvnTools(leap_path=self.leap_path, out_path=self.out_admin_path,
            #                             log=self.logger)
            #     svn_tool.write_svn_to_file()
            ### TO DO: Write git version tracker to replace this.

    def save_and_show(self, name, save=False, show=True, fig_num=None, clf=True, close=False, figsize=None, path=None):
        if save:
            if not hasattr(self, "out_path") or not os.path.isdir(self.out_path):
                self.create_output()
            if fig_num is not None:
                fig = plt.figure(fig_num)
            else:
                fig = plt.gcf()
            if figsize is None:
                figsize = (22, 14)
            fig.set_size_inches(figsize)
            if path is None:
                plt.savefig(os.path.join(self.out_path, name))
            else:
                plt.savefig(os.path.join(path, name))
        if show:
            plt.show()
        if clf:
            plt.clf()
        if close and not show:
            plt.close()

    def run_test(self):
        self.run()

    def end(self):
        time_str = times.time_string(time.time() - self.__start_time)
        self.logger.info(colored("==== finished %s (after %s) ====" % (self.__name, time_str), "cyan"))
        logging.shutdown()


def get_resources_path():
    return os.path.join(os.path.realpath(skywinder_analysis.__path__[0]), "resources")


if __name__ == "__main__":
    print("--- Doing simple test of App Class ---")
    app = App()



import os
import sys
import time

import matplotlib.animation as animation
import matplotlib.cm as cm
import matplotlib.pyplot as plt
import numpy as np
import scipy.stats
import scipy.ndimage
from skywinder_analysis.lib.tools import blosc_file
from skywinder_analysis.lib.image_processing import binning, flat_field
from skywinder_analysis.lib.skywinder_analysis_app import skywinder_analysis_app


class MovieMakerApp(skywinder_analysis_app.App):
    def ani_frame_from_raw(self):
        self.create_output()
        self.logger.info('Starting animation of frames')
        imd = np.zeros(self.settings.raw_image_size)
        self.logger.info('Section is %r' % self.settings.section)
        if self.settings.section:
            imd = imd[self.settings.section[0]:self.settings.section[1],
                  self.settings.section[2]:self.settings.section[3]]
        frames = len(self.settings.filenames)
        self.logger.info('%d filenames passed' % frames)
        fig = plt.figure()
        ax = fig.add_subplot(111)
        ax.get_xaxis().set_visible(False)
        ax.get_yaxis().set_visible(False)
        fig.subplots_adjust(left=0, bottom=0, right=1, top=1, wspace=None, hspace=None)
        if self.settings.bin_pixels:
            img = binning.bucket(imd, self.settings.bin_pixels)
        else:
            img = imd
        if self.settings.rotate:
            self.logger.info('Shape before: %d by %d' % (img.shape[0], img.shape[1]))
            img = np.rot90(img, k=1)
            self.logger.info('Shape after: %d by %d' % (img.shape[0], img.shape[1]))
        im = ax.imshow(img, cmap=cm.inferno, interpolation='nearest')
        self.logger.info('Image shape: %d by %d' % (img.shape[0], img.shape[1]))
        vsize = img.shape[0] / self.settings.dpi
        hsize = img.shape[1] / self.settings.dpi
        self.logger.info('Size is vertical: %.1f horizontal: %.1f' % (vsize, hsize))
        fig.set_size_inches([hsize, vsize])
        xfig, yfig = fig.get_dpi() * fig.get_size_inches()
        fontsize = yfig / 32
        txt = ax.text(0.01 * xfig, 0.05 * yfig, 'time:', ha='left', va='top', color='black', fontsize=fontsize)
        start_at = time.time()
        ax = plt.gca()
        if self.settings.saved_flat_field_path:
            self.logger.info('Loading saved flat field %s' % self.settings.saved_flat_field_path)
            flat_field_image = np.load(self.settings.saved_flat_field_path)
            use_flat_field_image = True
        elif self.settings.flat_field_filenames:
            self.logger.info('Generating flat field from %d files' % len(self.settings.flat_field_filenames))
            flat_field_image = flat_field.generate_flat_field_image_from_filenames(self.settings.flat_field_filenames)
            use_flat_field_image = True
            # Generate flat_field_image from filenames
        else:
            self.logger.info('No flat field used')
            use_flat_field_image = False

        def update_img(n):
            elapsed = time.time() - start_at
            if n:
                time_per_frame = elapsed / n
            else:
                time_per_frame = 1
            sys.stdout.flush()
            try:
                img, _ = blosc_file.load_blosc_image(self.settings.filenames[n])
                if use_flat_field_image:
                    img = flat_field.apply_flat_field(img, flat_field_image)
            except Exception as e:
                self.logger.info(e)
                return
            if self.settings.section:
                img = img[self.settings.section[0]:self.settings.section[1],
                      self.settings.section[2]:self.settings.section[3]]
            if self.settings.bin_pixels:
                img = binning.bucket(img, self.settings.bin_pixels)
            if self.settings.gaussian_filter_sigma:
                img = scipy.ndimage.gaussian_filter(img, sigma=self.settings.gaussian_filter_sigma)
            if self.settings.rotate:
                img = np.rot90(img, k=1)
            im.set_data(img)

            # if vmin == 0:
            vmin = scipy.stats.scoreatpercentile(img, 0.1)
            vmax = scipy.stats.scoreatpercentile(img, 99.9)
            vmin = scipy.stats.scoreatpercentile(
                img[self.settings.level_region[0]:self.settings.level_region[1],
                self.settings.level_region[2]:self.settings.level_region[3]],
                self.settings.min_percentile)
            vmax = scipy.stats.scoreatpercentile(
                img[self.settings.level_region[0]:self.settings.level_region[1],
                self.settings.level_region[2]:self.settings.level_region[3]],
                self.settings.max_percentile)
            im.set_clim(vmin, vmax)

            try:
                time_string = self.settings.filenames[n].split('/')[-1].split('_')[1]
                # Switch from Eastern time to UT
                hour = int(time_string[0:2])
                hour += 4
                hour = hour % 24

                time_string = ('Camera %d ' % self.settings.camera_number) + ('%d' % hour) + ':' + time_string[
                                                                                                   2:4] + ':' + time_string[
                                                                                                                4:]
                txt.set_text(time_string)
            except Exception:
                pass
            self.logger.info("\r%d of %d %.1f minutes elapsed, %.1f minutes remaining" % (
                n, len(self.settings.filenames), elapsed / 60,
                (len(self.settings.filenames) - n) * time_per_frame / 60))

            return im, txt

        ani = animation.FuncAnimation(fig, update_img, frames, interval=50)

        ani_fn = os.path.join(self.out_path, self.settings.output_name)
        ani.save(ani_fn, writer=self.settings.writer, dpi=self.settings.dpi)
        return ani


if __name__ == "__main__":
    app = MovieMakerApp()
    app.ani_frame_from_raw()
    app.end()



import scipy.io, scipy.signal, scipy.ndimage
import numpy.fft as fft
import numpy


def window2d(width, height, window_name='hann'):
    # Makes a circular Hanning window
    win = scipy.signal.get_window(window_name, height)

    half_width = (width - 1) / 2.0
    xx = numpy.linspace(-half_width, half_width, width)

    half_height = (height - 1) / 2.0
    yy = numpy.linspace(-half_height, half_height, height)

    x, y = numpy.meshgrid(xx, yy)
    radius_matrix = numpy.sqrt(x ** 2 + y ** 2)
    window_matrix = numpy.zeros(radius_matrix.shape)

    window_matrix[radius_matrix <= half_height] = \
        numpy.interp(radius_matrix[radius_matrix <= half_height], yy, win)

    return window_matrix


def get_1d_psd(data, window_name='hann'):
    win = scipy.signal.get_window(window_name, len(data))
    data = data * win

    data = fft.fft(data)
    data = fft.fftshift(data)
    data = numpy.abs(data) ** 2
    return data


def get_2d_psd(image):
    height, width = image.shape
    hanning_window = window2d(width, height)
    image = image * hanning_window

    data = fft.fft2(image)
    data = fft.fftshift(data)
    phase = numpy.angle(data)
    data = numpy.abs(data) ** 2
    return data


def get_wavelength_labels(physical_indices, freq_step, multiplier):
    values = []
    labels = []
    for i in physical_indices:
        if i == 0:
            values.append(numpy.nan)
        else:
            values.append(multiplier/(i*freq_step))
    for value in values:
        if numpy.isnan(value):
            labels.append('Inf')
        else:
            labels.append('%.1f' % value)
    print(labels)
    return labels


def get_freq_labels(physical_indices, freq_step, multiplier):
    values = []
    for i in physical_indices:
        values.append(i*freq_step*multiplier)
    labels = ['%.1f' % value for value in values]
    return labels


def get_2d_psd_and_labels(image, m_per_pixel, min_physical_range=5e3, tick_spacing=2, wavelength_labels=False):
    # The pixels are assumed to be square!
    T = m_per_pixel  # sampling interval

    ycenter = image.shape[0]//2
    xcenter = image.shape[1]//2

    psd = fft.fft2(numpy.nan_to_num(image-numpy.nanmean(image)))
    psd = fft.fftshift(psd)

    xN = image.shape[1]
    xfreq_scale = numpy.linspace(0, 1 / T, xN)
    xfreq_step = xfreq_scale[1]
    xmax_physical_index = numpy.searchsorted(xfreq_scale, 1/min_physical_range)+1
    xphysical_indices = range(0, xmax_physical_index, tick_spacing)
    xmaxrange = max(xphysical_indices)

    xindices = [xi + xcenter for xi in xphysical_indices]

    yN = image.shape[0]
    yfreq_scale = numpy.linspace(0, 1 / T, yN)
    yfreq_step = yfreq_scale[1]
    ymax_physical_index = numpy.searchsorted(yfreq_scale, 1/min_physical_range)+1
    yphysical_indices = range(-ymax_physical_index, ymax_physical_index, tick_spacing)
    ymaxrange = max(yphysical_indices)

    yindices = [yi + ycenter for yi in yphysical_indices]

    if wavelength_labels:
        xlabels = get_wavelength_labels(xphysical_indices, xfreq_step, 1e-3)
        ylabels = get_wavelength_labels(yphysical_indices, yfreq_step, 1e-3)
    else:
        xlabels = get_freq_labels(xphysical_indices, xfreq_scale[1], 1e5)
        ylabels = get_freq_labels(yphysical_indices, yfreq_scale[1], 1e5)
    return psd, xfreq_step, yfreq_step, xcenter, ycenter, xmaxrange, ymaxrange, xindices, yindices, xlabels, ylabels


def filter_psd_by_freq(psd, xfreq_step, yfreq_step, low_freq, high_freq):
    x,y = numpy.meshgrid(numpy.arange(psd.shape[1]),numpy.arange(psd.shape[0]))
    x_center = psd.shape[1]/2
    y_center = psd.shape[0]/2
    R = numpy.sqrt(((x-x_center)**2+((yfreq_step/xfreq_step)*(y-y_center))**2))
    Rmin = low_freq/xfreq_step
    Rmax = high_freq/xfreq_step
    filtered_psd = numpy.where((R >= Rmin) & (R <= Rmax),psd,0)
    # Finds where the psd array is between the R values and sets all other elements to zero.
    return filtered_psd



class Class(object):
    def __init__(self, **kwargs):
        for key in kwargs:
            setattr(self, key, kwargs[key])

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, str(self.__dict__))



import numpy as np
from matplotlib import pyplot as plt
import cv2
from collections import namedtuple
import scipy.optimize

class Blob(object):
    def __init__(self,x=None,y=None,peak=None,sigma=None):
        self.x = x
        self.y = y
        self.peak = peak
        self.sigma = sigma
    def __repr__(self):
        return 'Blob(x=%r,y=%r,peak=%r,sigma=%r)' % (self.x, self.y, self.peak, self.sigma)

def find_blobs(image,threshold=1000,cell_size=32,max_blobs_per_cell=1):
    dilated_image = cv2.dilate(image,cv2.getStructuringElement(cv2.MORPH_DILATE,(3,3)))
    #dilated_cells = dilated_image.reshape((image.shape[0]//cell_size,cell_size,image.shape[1]//cell_size,cell_size))
    cells = image.reshape((image.shape[0]//cell_size,cell_size,image.shape[1]//cell_size,cell_size))
    blobs = []
    for cellx in range(cells.shape[0]):
        for celly in range(cells.shape[2]):
            cell = cells[cellx,:,celly,:]
            possible_peaks = np.flatnonzero(cell > threshold)
            peaks = []
            for peak in possible_peaks:
                x,y = np.unravel_index(peak,(cell_size,cell_size))
                imagex = x + cellx*cell_size
                imagey = y + celly*cell_size
                if dilated_image[imagex,imagey] == image[imagex,imagey]: # true if we are at a local maximum
                    peaks.append(peak)
            peak_values = cell.flatten()[peaks]
            ordering = peak_values.argsort()
            peaks = np.array(peaks)
            for peak in peaks[ordering[-max_blobs_per_cell:]]:
                x,y = np.unravel_index(peak,(cell_size,cell_size))
                imagex = x + cellx*cell_size
                imagey = y + celly*cell_size
                blobs.append(Blob(imagex,imagey,image[imagex,imagey],None))
    return blobs


def gauss2d(params,xgrid,ygrid):
    A,x0,y0,sigmasq,offset = params
    gaussian = A*np.exp(-(((xgrid-x0)**2 + (ygrid-y0)**2)/(2*sigmasq))) + offset
    return gaussian

def objective(params,xgrid,ygrid,zdata):
    gaussian = gauss2d(params,xgrid,ygrid)
    return np.sum(np.abs((zdata-gaussian))**2)

def fit_gauss_2d(data,A0 = None, x0 = None, y0 = None, sigma0 = 1, offset = None):
    if x0 is None:
        x0 = data.shape[0]/2.0
        y0 = data.shape[1]/2.0
    if offset is None:
        offset = data.min()
    if A0 is None:
        A0 = data.ptp()
    init_params = (A0,x0,y0,sigma0**2,offset)
    ygrid,xgrid = np.meshgrid(np.arange(data.shape[1]),np.arange(data.shape[0]))
    return xgrid,ygrid,scipy.optimize.minimize(objective,init_params,args=(xgrid,ygrid,data))

class BlobFinder(object):
    def __init__(self, im, blob_threshold=6, kernel_size=8, kernel_sigma=2, fitting_region_size=32, fit_blobs=True,
                 cell_size=128, max_blobs_per_cell=1, rmax=4):

        kernel = cv2.getGaussianKernel(kernel_size,sigma=kernel_sigma)
        kernel = np.outer(kernel,kernel)
        # zero out the pixels that will make the "trough" before normalizing so that the mean will end up being zero
        # after adding the trough
        kernel[0,:] = 0
        kernel[:,0] = 0
        kernel[-1,:] = 0
        kernel[:,-1] = 0
        kernel = kernel/kernel.sum()

        # npix is the total number of pixels in the trough
        npix = kernel.shape[0]*4-2
        kernel[0,:] = -1.0/npix
        kernel[:,0] = -1.0/npix
        kernel[-1,:] = -1.0/npix
        kernel[:,-1] = -1.0/npix
        self.kernel = kernel
        #convolve
        imlev =  cv2.filter2D(im.astype('float'),-1,kernel)

        # Make sure that the leveled image can be divided evenly by the cell_size
        nr = (imlev.shape[0]//cell_size)*cell_size
        nc = (imlev.shape[1]//cell_size)*cell_size
        # leveled is the filtered image that is an even multiple of the cell_size
        self.leveled = imlev[:nr,:nc]
        cells = self.leveled.reshape((imlev.shape[0]//cell_size,cell_size,
                                        imlev.shape[1]//cell_size,cell_size))
        #calculate noise
        noise = np.median(cells.std(axis=(1,3)))
        self.global_noise = noise
        self.cells = cells

        #find blobs, now using local MAD noise estimate. threshold is in units of MAD
        blobs = self.find_blobs(threshold=blob_threshold,cell_size=cell_size,
                                max_blobs_per_cell=max_blobs_per_cell, rmax=rmax)
        #fit blobs
        if fit_blobs:
            for blob in blobs:
                xmin = np.max((blob.x - fitting_region_size // 2, 0))
                xmax = np.min((blob.x + fitting_region_size // 2, im.shape[0] - 1))
                ymin = np.max((blob.y - fitting_region_size // 2, 0))
                ymax = np.min((blob.y + fitting_region_size // 2, im.shape[1] - 1))

                stamp = im[xmin:xmax,ymin:ymax]
                xgrid,ygrid,result = fit_gauss_2d(stamp)
                blob.sigma = result.x[3]
        self.blobs = blobs
    def find_blobs(self,threshold=6,cell_size=32,max_blobs_per_cell=1, rmax=4):
        """

        Parameters
        ----------
        threshold
        cell_size
        max_blobs_per_cell
        rmax : radius in pixels for two blobs to be considered the same object

        Returns
        -------

        """
        rsquared = rmax**2

        cells = self.leveled.reshape((self.leveled.shape[0]//cell_size,cell_size,self.leveled.shape[1]//cell_size,cell_size))
        blobs = []
        for cellx in range(cells.shape[0]):
            for celly in range(cells.shape[2]):
                cell = cells[cellx,:,celly,:]

                median = np.median(cell)
                deviations = cell-median
                cell_mad = np.median(np.abs(deviations))
                possible_peaks = np.flatnonzero(deviations > threshold*cell_mad)

                peakxy = []
                peak_values = cell.flatten()[possible_peaks]
                ordering = peak_values.argsort()[::-1]  #reverse the ordering because we want to order from largest
                # to smallest brightness
                possible_peaks = possible_peaks[ordering]
                # iterate through peaks in descending brightness order
                for peak in possible_peaks:
                    x,y = np.unravel_index(peak,(cell_size,cell_size))
                    if len(peakxy):
                        pxy = np.array(peakxy)
                        distances = (pxy[:,0]-x)**2 + (pxy[:,1]-y)**2
                        if np.any(distances<rsquared):
                            # this peak is close to an already found peak, so we skip it
                            continue
                    imagex = x + cellx*cell_size
                    imagey = y + celly*cell_size
                    blobs.append(Blob(imagex,imagey,self.leveled[imagex,imagey],None))
                    if len(blobs) >= max_blobs_per_cell:
                        break
        #unify blobs that were detected in more than one cell
        blobs.sort(key=lambda x:x.peak,reverse=True)
        bxy = []
        unique_blobs = []
        for blob in blobs:
            if len(bxy):
                pxy = np.array(bxy)
                distances = (pxy[:,0]-blob.x)**2 + (pxy[:,1]-blob.y)**2
                if np.any(distances<rsquared):
                    continue
            bxy.append((blob.x,blob.y))
            unique_blobs.append(blob)
        return unique_blobs



from skywinder_analysis.lib.tools import generic
import matplotlib.animation as animation
import matplotlib.cm as cm

settings = generic.Class()

settings.section = None
settings.bin_pixels = (4, 4)
settings.dpi = 100

settings.cameras = [4, 5, 6, 7]

settings.raw_image_size = (3232, 4864)
settings.output_name = 'movie.mp4'
settings.all_filenames = []  # use glob to make list of lists of filenames
settings.writer = animation.writers['ffmpeg'](fps=10, codec='h264', bitrate=2 ** 20)
settings.level_region = [4864 // (4 * settings.bin_pixels[0]),
                         (3 * 4864) // (4 * settings.bin_pixels[0]),
                         (3232 * 2) // settings.bin_pixels[1],
                         (3232 * 3) // settings.bin_pixels[1]]
# Level full stitched by central part of camera 6

settings.crop = False

settings.saved_flat_field_paths = False
settings.all_flat_field_filenames = False
settings.use_new_flat_field = False
settings.new_flat_field_reflection_window = 3000

settings.mean_region = [0 // settings.bin_pixels[0],
                        3232 // settings.bin_pixels[0],
                        4864 // (4 * settings.bin_pixels[1]),
                        (3 * 4864) // (4 * settings.bin_pixels[1])]
# Compare mean of images using central 50%

settings.color_scheme = cm.inferno
# Color scheme of image
settings.fontcolor = 'black'

settings.colorscale_by_frame = True
settings.vmin = False
settings.vmax = False
settings.individual_frames = False
settings.split_video_len = False
settings.font_proportion = 1./32.
settings.no_text = False
settings.rotate = False
settings.piggyback = False



import numpy as np
from numpy.lib.stride_tricks import as_strided
import warnings


def view_as_blocks(arr_in, block_shape):
    if not isinstance(block_shape, tuple):
        raise TypeError('block needs to be a tuple')

    block_shape = np.array(block_shape)
    if (block_shape <= 0).any():
        raise ValueError("'block_shape' elements must be strictly positive")

    if block_shape.size != arr_in.ndim:
        raise ValueError("'block_shape' must have the same length "
                         "as 'arr_in.shape'")

    arr_shape = np.array(arr_in.shape)
    if (arr_shape % block_shape).sum() != 0:
        raise ValueError("'block_shape' is not compatible with 'arr_in'")

    # -- restride the array to build the block view

    if not arr_in.flags.contiguous:
        warnings.warn(RuntimeWarning("Cannot provide views on a non-contiguous input "
                                     "array without copying."))

    # Taken from https://github.com/scikit-image/scikit-image/blob/master/skimage/util/shape.py#l10
    arr_in = np.ascontiguousarray(arr_in)

    new_shape = tuple(arr_shape // block_shape) + tuple(block_shape)
    new_strides = tuple(arr_in.strides * block_shape) + arr_in.strides

    arr_out = as_strided(arr_in, shape=new_shape, strides=new_strides)

    return arr_out


def bucket(x, bucket_size):
    # from https://stackoverflow.com/questions/36269508/lets-make-a-reference-implementation-of-n-dimensional-pixel-binning-bucketing-f
    blocks = view_as_blocks(x, bucket_size)
    tup = tuple(range(-len(bucket_size), 0))
    # return blocks.sum(axis=tup)
    return blocks.mean(axis=tup)



from default_settings import settings
import glob
import os
from pmc_analysis.lib.tools import periodic_image_finder
import numpy as np

settings.hot_pixel_dirs = glob.glob('/data/home/bjorn/pmc-turbo/config/camera_data/hot_pixels/*.npy')
settings.image_shape = (3232,4864)

n = 1531402200 - (30*60)# 2018-07-12_1300
hour_sec = 3600
settings.starts = np.arange(-hour_sec*48, hour_sec*24, hour_sec) + n
settings.duration = 120

settings.camera_number = 4
settings.new_flat_field_window = 600

settings.all_filename_lists = []
for start in settings.starts:
    full_fns = []

    fn_list = periodic_image_finder.find_all_images(settings.camera_number,
                                                    start_timestamp=start,
                                                    stop_timestamp=start+settings.duration)
    if settings.camera_number > 4:
        direc = '/data/mounted_filesystems/nas2/c{0}'.format(settings.camera_number)
    else:
        direc = '/data/mounted_filesystems/nas1/c{0}'.format(settings.camera_number)
    for fn in fn_list:
        full_fns.append(os.path.join(direc, fn))


    settings.all_filename_lists.append(full_fns)

settings.pointing_solutions_dir = '/data/mounted_filesystems/nas2/resources/pointing_solutions/c%d_2018-07-12_2330_solution.csv' % settings.camera_number

settings.min_blob_trajectory_length = len(settings.all_filename_lists[0])//2



from default_settings import settings
from pmc_analysis.lib.tools import periodic_image_finder
import glob

n = 1578643200 + (9*60*60)
settings.output_name = '2020-01-10_1700.mp4'
start = n
stop = start + (30*60)
interval = 4

settings.bin_pixels = (4, 4)

settings.rolling_flat_field_size = 600 // interval
settings.filenames = periodic_image_finder.find_periodic_images_full_fn(99, start, stop, interval)

settings.level_region = [3232//(4 * settings.bin_pixels[0]), (3 * 3232)//(4 * settings.bin_pixels[0]), 0//settings.bin_pixels[1], 4864//settings.bin_pixels[1]]

#settings.filenames = glob.glob('/data/pmc_turbo_raw_data/c4/2018-07-13/2018*')[::10]
#settings.level_region = [0//settings.bin_pixels[0],3232//settings.bin_pixels[0],(4864//4)//settings.bin_pixels[1],(3*4864//4)//settings.bin_pixels[1]]



import astropy
from astropy import units as u
from astropy.coordinates import EarthLocation, SkyCoord, ICRS, AltAz
from astropy.time import Time

CSBF_LOCATION = EarthLocation(lat=31.779690 * u.deg, lon=-95.716 * u.deg)


def ra_dec_to_alt_az(ra, dec, time, location=CSBF_LOCATION):
    radec_corner = SkyCoord(ra=ra * u.deg, dec=dec * u.deg)
    altaz_corner = radec_corner.transform_to(AltAz(obstime=time, location=location))
    return altaz_corner



import pandas as pd
import os
import numpy as np
import scipy.interpolate as interpolate


def create_interpolation_fns(cam_num, pointing_directory):
    fn = os.path.join(pointing_directory, 'c{0}_coordinates.csv'.format(cam_num))
    x_original = np.arange(0, 3232, 34)
    y_original = np.arange(0, 4864, 34)
    HW = np.loadtxt(fn, delimiter=',')


def run_grid_map(fn, cam_num, bin_factor, pointing_directory):
    df = pd.read_csv(
        os.path.join(pointing_directory, ('c%d_2018-07-12_2330_solution.csv' % cam_num)))


    # f_h = interpolate.interp2d(df.az, df.alt, df.h,
    #                            kind='cubic', bounds_error=False, fill_value=np.nan)
    # f_w = interpolate.interp2d(df.az, df.alt, df.w,
    #                            kind='cubic', bounds_error=False, fill_value=np.nan)

    # Get pixel coordinates from x, y on sky
    h = f_h(x, y)
    w = f_w(x, y)




import skywinder_analysis.lib.tools.blosc_file as blosc_file

def get_dark_image(camera_number, exposure):
    if camera_number == 99:
        camera_number = 7
    dark_100ms, _ = blosc_file.load_blosc_image('/home/christopher/SkyWinder-Analysis/skywinder_analysis/resources/dark_images/c{0}_dark_image_100ms'.format(camera_number))
    dark_1s, _ = blosc_file.load_blosc_image('/home/christopher/SkyWinder-Analysis/skywinder_analysis/resources/dark_images/c{0}_dark_image_1s'.format(camera_number))    
    dark = dark_100ms.astype('float') + float((exposure - 100000)/900000)*(dark_1s.astype('float') - dark_100ms.astype('float'))
    return dark



import glob
import os
import skywinder_analysis


def find_periodic_images(camera_number, start_timestamp, stop_timestamp, interval):
    all_filenames = get_all_image_filenames(camera_number)
    fn_list = []
    current_timestamp = start_timestamp
    for fn in all_filenames:
        try:
            if current_timestamp <= int(fn.split('=')[-1][:10]):
                fn_list.append(fn.strip('\n'))
                current_timestamp += interval
                if current_timestamp >= stop_timestamp:
                    break
        except Exception as e:
            print(fn)
            raise(e)
    return fn_list


def find_periodic_images_full_fn(camera_number, start_timestamp, stop_timestamp, interval):
    all_filenames = get_all_image_filenames(camera_number)
    fn_list = []
    current_timestamp = start_timestamp
    for fn in all_filenames:
        if len(fn.split('=')) > 1:
            if current_timestamp <= int(fn.split('=')[-1][:10]):
                if int(fn.split('=')[-1][:10]) > stop_timestamp:
                    break
                fn_list.append(fn.strip('\n'))
                current_timestamp += interval
                if current_timestamp >= stop_timestamp:
                    break
    full_fns = []
    if camera_number < 4:
        direc = '/data/mounted_filesystems/nas1/c{0}'.format(camera_number)
    elif camera_number == 99:
        direc = '/data/mounted_filesystems/nas1/piggyback'
    else:
        direc = '/data/mounted_filesystems/nas2/c{0}'.format(camera_number)
    for fn in fn_list:
        full_fns.append(os.path.join(direc, fn))

    return full_fns

def find_all_images(camera_number, start_timestamp, stop_timestamp):
    all_filenames = get_all_image_filenames(camera_number)
    fn_list = []
    for fn in all_filenames:
        if start_timestamp <= int(fn.split('=')[-1][:10]):            
            if int(fn.split('=')[-1][:10]) >= stop_timestamp:
                break
            fn_list.append(fn.strip('\n'))
    return fn_list


INDIVIDUAL_CAMERA_DIRNAMES = {
    '1': '2018-07-08_035140',
    '2': '2018-07-07_100519',
    '3': '2018-07-07_100509',
    '4': '2018-07-07_100519',
    '6': '2018-07-07_100513',
    '7': '2018-07-08_035010'}


#def get_fn_end(fn):
#    return fn.split('/')[-1]

def get_fn_end(fn):
    return fn.split('=')[-1]


def get_all_image_filenames(camera_number):
    all_filenames = []
    if camera_number == 5:
        # Camera 5 was restarted and thus has two sets of filenames and directories
        for i in range(1, 6):
            for date in ['2018-07-07_100515', '2018-07-13_031421']:
                with open(os.path.join(skywinder_analysis.__path__[0], 'resources', 'new_list_of_all_filenames', 'c5',
                                       ('c5_data%d_%s.txt' % (i, date)))) as f:
                    fns = f.readlines()
                    for fn in fns:
                        all_filenames.append(
                            os.path.join(('data%d' % i), date, fn))
    elif camera_number == 99:
        # Piggyback is "camera 99"
        # Piggyback was restarted and thus has two sets of filenames and directories
        for i in range(1, 6):
            for date in ['2019-12-15_144434', '2019-12-19_012658']:
                with open(os.path.join(skywinder_analysis.__path__[0], 'resources', 'list_of_all_filenames', 'c99',
                                       ('c99_data%d_%s.txt' % (i, date)))) as f:
                    fns = f.readlines()
                    for fn in fns:
                        all_filenames.append(
                            os.path.join(('data%d' % i), date, fn))
    else:
        # Cameras 1, 2, 3, 4, 6, 7
        for i in range(1, 6):
            with open(os.path.join(skywinder_analysis.__path__[0], 'resources', 'new_list_of_all_filenames',
                                   ('c%d' % camera_number),
                                   ('c%d_data%d.txt' % (camera_number, i)))) as f:
                fns = f.readlines()
                for fn in fns:
                    all_filenames.append(
                        os.path.join(('data%d' % i), INDIVIDUAL_CAMERA_DIRNAMES[str(camera_number)], fn))
    all_filenames.sort(key=get_fn_end)
    return all_filenames



from default_settings import settings
import matplotlib.animation as animation
import os
from pmc_analysis.lib.tools import periodic_image_finder
import matplotlib.cm as cm
import datetime
import calendar

#n = 1577754600 # 2019-12-31_0110
#settings.output_name = '2019-12-19_0110_v_shapes.mp4'
#start = n
#stop = start + (10*60)

#n = 1578136740 # 2020 01 04 0619
n = 1578599460 # 2020 01 09 1451
start = n
stop = start + (15*60) 
interval = 10

filenames = periodic_image_finder.find_periodic_images_full_fn(99, start, stop, interval)
settings.all_filenames = [filenames]
settings.cameras = [99]

#settings.level_region = [3232//(4 * settings.bin_pixels[0]), (3 * 3232)//(4 * settings.bin_pixels[0]), 0//settings.bin_pixels[1], 4864//settings.bin_pixels[1]]
settings.level_region = [0, 4864//settings.bin_pixels[0], 3232//(4*settings.bin_pixels[0]), (3*3232)//(4*settings.bin_pixels[0])]

settings.use_new_flat_field = True
settings.new_flat_field_reflection_window = 600
settings.all_flat_field_filenames = settings.all_filenames


settings.bin_pixels = (8,8)

#settings.level_region = [4864 // (4 * settings.bin_pixels[0]),
#                         (4864) // (settings.bin_pixels[0]),#(3 * 4864) // (4 * settings.bin_pixels[0]),
#                         0 // settings.bin_pixels[1],
#                         (len(settings.all_filenames) * 3232) // settings.bin_pixels[1]]
# Level full stitched by central part of all cameras



settings.color_scheme = cm.inferno
#settings.fontcolor = 'white'
settings.crop = False #[3232//4,3232,4864//4,5*4864//8]
settings.colorscale_by_frame = True
settings.vmin = False
settings.vmax = False

settings.individual_frames = True
#settings.split_video_len = (20*60 // interval)
settings.font_proportion = 1./16.
settings.no_text = False

settings.piggyback = True

settings.rotate = True
settings.rotate_k = -1



from skywinder_analysis.lib.pointing import blobs
import matplotlib.pyplot as plt
import numpy as np


def filter_low_contrast_blobs(image, blobs, m=1, debug=False):
    good_blobs = []
    mask = [[0, 0, 0], [0, 1, 0], [0, 0, 0]]
    for blob in blobs:
        tiny_stamp = image[blob.x - 2:blob.x + 1, blob.y - 2:blob.y + 1]
        if tiny_stamp.shape == (3, 3):
            tiny_stamp = np.ma.masked_array(tiny_stamp, mask=mask)
            stamp = image[blob.x - 16:blob.x + 16, blob.y - 16:blob.y + 16]
            if debug:
                print('Local mean:', np.mean(tiny_stamp))
                print('Stamp mean:', np.mean(stamp))
                print('Stamp std:', np.std(stamp))
                print('Difference in means', np.mean(tiny_stamp) - np.mean(stamp))

            if np.mean(tiny_stamp) > (np.mean(stamp) + m * np.std(stamp)):
                good_blobs.append(blob)
    return good_blobs


def find_blobs(image, show_blobs=True, fit_blobs=True, do_low_contrast_filter=True, m=1):
    bf = blobs.BlobFinder(image, fit_blobs=fit_blobs)  # ,cell_size=64,blob_threshold=12,fit_blobs=False,kernel_sigma=1)
    myblobs = bf.blobs
    if do_low_contrast_filter == True:
        myblobs = filter_low_contrast_blobs(image, myblobs, m, debug=False)
    if show_blobs == True:
        fig = plt.figure(figsize=(9, 9))
        axs = fig.add_subplot(4, 8)
        for k, ax in enumerate(axs.flatten()):
            if k >= len(myblobs):
                break
            blob = myblobs[k]
            ax.imshow(image[blob.x - 16:blob.x + 16, blob.y - 16:blob.y + 16], cmap=plt.cm.hot, interpolation='nearest')
            ax.set_title("X: %d\nY: %d" % (blob.x, blob.y))
    return myblobs



from default_settings import settings
import numpy as np
import matplotlib.animation as animation
import matplotlib.cm as cm


import os
from pmc_analysis.lib.tools import periodic_image_finder
#n = 1531190700 # 2018 7 10 02 45
n = 1531402200 # 2018 7 12 13 30

start = n - (10*60)
stop = n + (10*60)

#start = n
#stop = n + 30 

#start = 1531285200 # 2018-07-11_0500
#stop = start + (60*60)
interval = 30

#settings.camera_numbers = [4,5,6,7]
#settings.all_filename_lists = {4: [], 5:[], 6:[], 7:[]}
settings.cam_nums = [5,6]
settings.all_filename_lists = {5:[], 6:[]}
for cam_num in settings.cam_nums:
    filenames = periodic_image_finder.find_periodic_images_full_fn(cam_num,
                                                                            start_timestamp=start,
                                                                            stop_timestamp=stop,
                                                                            interval=interval)

    settings.all_filename_lists[cam_num] = filenames

settings.min_x = -50
settings.max_x = 50
settings.x_increment = 1000

settings.min_y = 10
settings.max_y = 110
settings.y_increment = 1000

settings.bin_factor = 4

settings.xvalues = np.linspace(settings.min_x, settings.max_x, num=settings.x_increment)
settings.yvalues = np.linspace(settings.min_y, settings.max_y, num=settings.y_increment)
settings.individual_frames = True
settings.grid = True
settings.grid_spacing = 5
#settings.color_scheme = cm.Blues_r

settings.timestamp_in_image = True


settings.m_per_pixel = 100
settings.min_physical_range = 5e3
settings.tick_spacing = 1

settings.individual_frames = True
settings.wavelength_labels = True
settings.filter_freq = False
settings.low_freq_filter = 1/60e3
settings.high_freq_filter = 1/14e3
settings.text_color = 'white'



from default_settings import settings
import numpy as np
import matplotlib.animation as animation


import os
from pmc_analysis.lib.tools import periodic_image_finder
n = 1531402200 # 2018-07-12_1330
start = n
stop = start + 2
interval = 2

settings.cam_nums = [5]
settings.all_filename_lists = {5:[]}
for cam_num in settings.cam_nums:
    filenames = periodic_image_finder.find_periodic_images_full_fn(cam_num,
                                                                            start_timestamp=start,
                                                                            stop_timestamp=stop,
                                                                            interval=interval)

    settings.all_filename_lists[cam_num] = filenames

settings.min_x = -100 + 45
settings.max_x = settings.min_x + (12*5)
settings.x_increment = 500

settings.min_y = 20
settings.max_y = settings.min_y + (18*5)
settings.y_increment = 750

settings.bin_factor = 8

settings.xvalues = np.linspace(settings.min_x, settings.max_x, num=settings.x_increment)
settings.yvalues = np.linspace(settings.min_y, settings.max_y, num=settings.y_increment)
settings.individual_frames = True
settings.grid = True
settings.grid_spacing = 5



from default_settings import settings
import matplotlib.animation as animation


import os
from pmc_analysis.lib.tools import periodic_image_finder
#n = 1531402200 # 2018-07-12_1330
#start = n - (60*60)
#stop = n + (15*60)
#interval = 10

#n = 1531298400 # 2018-07-11_0840
#start = n
#stop = start + (5*60)
#interval = 2

#n = 1531298400 # 2018-07-11_0840
#start = n + (25*60)
#stop = start + (10*60)
#interval = 2

n = 1531288200 # 2018-07-11_0550
start = n-(5*60)
stop = n + (10*60)
interval = 2

settings.camera_number = 3

settings.filenames = periodic_image_finder.find_periodic_images_full_fn(settings.camera_number,
                                                                        start_timestamp=start,
                                                                        stop_timestamp=stop,
                                                                        interval=interval)

settings.flat_field_filenames = settings.filenames[::10]
settings.writer = animation.writers['ffmpeg'](fps=20, codec='mpeg4', bitrate=2**20)
settings.bin_pixels = (4,4)
settings.rotate = True
#settings.section = [0,3232,0,4864]
settings.section = [0,3232,0,(5*4864)//8]



from default_settings import settings
import matplotlib.animation as animation


import os
from pmc_analysis.lib.tools import periodic_image_finder
n = 1531400400 # 2018-07-12_0900
start = n
end = start + (120*60)
interval = 2

settings.camera_number = 5

filenames = periodic_image_finder.find_periodic_images(settings.camera_number, start, end, interval)

joined_filenames = []
for fn in filenames:
        joined_filenames.append(os.path.join(('/data/mounted_filesystems/nas2/c%d/'%settings.camera_number), fn))
settings.filenames = joined_filenames

settings.flat_field_filenames = settings.filenames[::5]
settings.writer = animation.writers['ffmpeg'](fps=20, codec='mpeg4', bitrate=2**20)
settings.bin_pixels = (4,4)
settings.rotate = False



import glob
import os
import pandas as pd
import numpy as np
from scipy import interpolate as inter

import pmc_analysis


def check_focus(times):
    df1 = pd.read_csv('/data/mounted_filesystems/nas2/c5/var/pmclogs/housekeeping/camera/2018-07-07_100517.csv', comment='#')
    df2 = pd.read_csv('/data/mounted_filesystems/nas2/c5/var/pmclogs/housekeeping/camera/2018-07-13_031422.csv', comment='#')

    full_focus = np.concatenate((np.asarray(df1.EFLensFocusCurrent, dtype=np.float64),
                        np.asarray(df2.EFLensFocusCurrent, dtype=np.float64)))
    epoch = np.concatenate((np.asarray(df1.epoch, dtype=np.float64),
                        np.asarray(df2.epoch, dtype=np.float64)))
    f_focus = inter.interp1d(epoch, full_focus)

    focus = f_focus(times)
    focus_flag = np.ones(len(times))
    focus_flag[focus < 4000] = 0
    focus_flag[focus > 6000] = 0

    return focus_flag



import pandas as pd
import numpy as np
from scipy import interpolate as inter

from astropy import units as u
from astropy.coordinates import EarthLocation, AltAz, get_sun
from astropy.time import Time

def get_lat_long_alt_SIP(timestamp, df= pd.read_csv('/data/mounted_filesystems/nas2/resources/SIP_gps_plus_solar_data.csv')):
    #If calling this function repeatedly, reading in the csv separately and passing it will be more efficient. 
    #Returns latitude and longitude in degrees, altitude in kft

    if timestamp > 1532000000:
        df = pd.read_csv('/data/mounted_filesystems/nas2/resources/SIP_gps_plus_solar_data_piggyback_unwrapped.csv')
    
    i0 = np.where(df.epoch <= timestamp)[0][-1]
    i1 = np.where(df.epoch > timestamp)[0][0]
    
    alt = df.altitude[i0] + (timestamp - df.epoch[i0])/(df.epoch[i1] - df.epoch[i0])*(df.altitude[i1] - df.altitude[i0])
    lat = df.latitude[i0] + (timestamp - df.epoch[i0])/(df.epoch[i1] - df.epoch[i0])*(df.latitude[i1] - df.latitude[i0])
    long = df.longitude[i0] + (timestamp - df.epoch[i0])/(df.epoch[i1] - df.epoch[i0])*(df.longitude[i1] - df.longitude[i0])
    
    return lat, long, alt

def get_lat_long_alt_SIP_array(timestamps,
                               df= pd.read_csv('/data/mounted_filesystems/nas2/resources/SIP_gps_plus_solar_data.csv')):
    #If calling this function repeatedly, reading in the csv separately and passing it will be more efficient. 
    #Returns latitude and longitude in degrees, altitude in kft
    
    if timestamps[0] > 1532000000:
        df = pd.read_csv('/data/mounted_filesystems/nas2/resources/SIP_gps_plus_solar_data_piggyback_unwrapped.csv')
    f_lat = inter.interp1d(df.epoch, df.latitude)
    f_long = inter.interp1d(df.epoch, df.longitude)
    f_alt = inter.interp1d(df.epoch, df.altitude)
    
    
    alt = f_alt(timestamps)
    lat = f_lat(timestamps)
    long = f_long(timestamps)
    
    return lat, long, alt

def get_pointing_stars(timestamp, df = pd.read_csv('/data/mounted_filesystems/nas2/resources/star_pointing_data.csv')):
    #If calling this function repeatedly, reading in the csv separately and passing it will be more efficient. 
    #Returns azimuth pointing of center of the field of view in radians relative to N in CW direction
    #(i.e. pi/2 corresponds to E).
    if timestamp >= 1531100000:    
        i0 = np.where(df.epoch <= timestamp)[0][-1]
        i1 = np.where(df.epoch > timestamp)[0][0]
    
        if df.az[i0] > 330 and df.az[i1] < 30:
            az = df.az[i0] + (timestamp - df.epoch[i0])/(df.epoch[i1] - df.epoch[i0])*(df.az[i1] + 360 - df.az[i0])
            az = az % 360
        else:
            az = df.az[i0] + (timestamp - df.epoch[i0])/(df.epoch[i1] - df.epoch[i0])*(df.az[i1] - df.az[i0])
        return np.radians(az)
    else:
        az = get_pointing_rotator(timestamp)
        return az
    
def get_pointing_stars_array(timestamps, df = pd.read_csv('/data/mounted_filesystems/nas2/resources/star_pointing_data.csv')):
    if np.min(timestamps) >= 1531100000:
        f_az = inter.interp1d(df.epoch, df.az)
        az = f_az(timestamps)
        return np.radians(az)
    else:
        az = get_pointing_rotator_array(timestamps)
        return az

def get_pointing_SIP(timestamp, df = pd.read_csv('/data/mounted_filesystems/nas2/resources/SIP_yaw_data_cleaned.csv')):
    #If calling this function repeatedly, reading in the csv separately and passing it will be more efficient. 
    #Returns azimuth pointing of center of the field of view in radians relative to N in CW direction
    #(i.e. pi/2 corresponds to E).
    if timestamp > 1532000000:
        print('No SIP data available: use get_pointing_rotator')
        
    i0 = np.where(df.epoch <= timestamp)[0][-1]
    i1 = np.where(df.epoch > timestamp)[0][0]
    
    if df.yaw[i0] > 330 and df.yaw[i1] < 30:
        yaw = df.yaw[i0] + (timestamp - df.epoch[i0])/(df.epoch[i1] - df.epoch[i0])*(df.yaw[i1] + 360 - df.yaw[i0])
        yaw = yaw % 360
    else:
        yaw = df.yaw[i0] + (timestamp - df.epoch[i0])/(df.epoch[i1] - df.epoch[i0])*(df.yaw[i1] - df.yaw[i0])
    
    return np.radians(yaw)

def get_pointing_SIP_array(timestamps, df = pd.read_csv('/data/mounted_filesystems/nas2/resources/SIP_yaw_data_cleaned.csv')):
    if timestamps[0] > 1532000000:
        print('No SIP data available: use get_pointing_rotator_array')
    f_yaw = inter.interp1d(df.epoch, df.yaw)
    yaw = f_yaw(timestamps)
        
    return np.radians(yaw)

def get_pointing_rotator_array(timestamps,
                              df = pd.read_csv('/data/mounted_filesystems/nas2/resources/rotator_pointing_data_unwrapped.csv',
                                                     header=0, names=['epoch', 'az'])):
    #If calling this function repeatedly, reading in the csv separately and passing it will be more efficient. 
    #Returns azimuth pointing of center of the field of view in radians relative to N in CW direction
    #(i.e. pi/2 corresponds to E).
    
    if timestamps[0] > 1532000000:
        df = pd.read_csv('/data/mounted_filesystems/nas2/resources/rotator_pointing_data_piggyback_unwrapped.csv')
        f_az = inter.interp1d(df.epoch, df.az)
        az = np.radians(f_az(timestamps))
    
    else:
        f_az = inter.interp1d(df.epoch, df.az)
        az_offset = f_az(timestamps)
    
        sun_az, sun_alt = get_sun_az_alt_array(timestamps)
        az = sun_az + np.radians(az_offset) - np.pi
    
    return az

def get_pointing_rotator(timestamp, df = pd.read_csv('/data/mounted_filesystems/nas2/resources/rotator_pointing_data.csv',
                                                     header=0, names=['epoch', 'az'])):
    #If calling this function repeatedly, reading in the csv separately and passing it will be more efficient. 
    #Returns azimuth pointing of center of the field of view in radians relative to N in CW direction
    #(i.e. pi/2 corresponds to E).
    if timestamp > 1532000000:
        df = pd.read_csv('/data/mounted_filesystems/nas2/resources/rotator_pointing_data_piggyback_unwrapped.csv')
    
    i0 = np.where(df.epoch <= timestamp)[0][-1]
    i1 = np.where(df.epoch > timestamp)[0][0]
    
    if df.az[i0] > 150 and df.az[i1] < -150:
        az_offset = df.az[i0] + (timestamp - df.epoch[i0])/(df.epoch[i1]
                                                            - df.epoch[i0])*(df.az[i1] + 360 - df.az[i0])
        az_offset = az_offset % 360
    else:
        az_offset = df.az[i0] + (timestamp - df.epoch[i0])/(df.epoch[i1] -
                                                            df.epoch[i0])*(df.az[i1] - df.az[i0])
    if timestamp > 1532000000:
        az = np.radians(az_offset)
    else:
        sun_az, sun_alt = get_sun_az_alt(timestamp)
        az = sun_az + np.radians(az_offset) - np.pi
    
    return az

def get_sun_az_alt(timestamp, df= pd.read_csv('/data/mounted_filesystems/nas2/resources/SIP_gps_plus_solar_data.csv')):  
    #If calling this function repeatedly, reading in the csv separately and passing it will be more efficient. 
    #Returns the azimuth and altitude of the sun in radians at the location of the gondola.

    if timestamp > 1532000000:
        df = pd.read_csv('/data/mounted_filesystems/nas2/resources/SIP_gps_plus_solar_data_piggyback_unwrapped.csv')
    
    i0 = np.where(df.epoch < timestamp)[0][-1]
    i1 = np.where(df.epoch > timestamp)[0][0]
    if (df.sun_az[i0] > 330 and df.sun_az[i1] < 30):
        az = df.sun_az[i0] + (timestamp - df.epoch[i0])/(df.epoch[i1] - df.epoch[i0])*(df.sun_az[i1] + 360 - df.sun_az[i0])
    else:
        az = df.sun_az[i0] + (timestamp - df.epoch[i0])/(df.epoch[i1] - df.epoch[i0])*(df.sun_az[i1] - df.sun_az[i0])
    az = az % 360
    
    alt = df.sun_alt[i0] + (timestamp - df.epoch[i0])/(df.epoch[i1] - df.epoch[i0])*(df.sun_alt[i1] - df.sun_alt[i0])
    
    return np.radians(az), np.radians(alt)

def get_sun_az_alt_array(timestamps):
    if timestamps[0] < 1532000000:
        df = pd.read_csv('/data/mounted_filesystems/nas2/resources/SIP_gps_plus_solar_data_unwrapped.csv')
    else:
        df = pd.read_csv('/data/mounted_filesystems/nas2/resources/SIP_gps_plus_solar_data_piggyback_unwrapped.csv')
    f_sun_az = inter.interp1d(df.epoch, df.sun_az)
    f_sun_alt = inter.interp1d(df.epoch, df.sun_alt)
    
    return np.radians(f_sun_az(timestamps)), np.radians(f_sun_alt(timestamps))

def get_moon_az_alt(timestamp, df= pd.read_csv('/data/mounted_filesystems/nas2/resources/SIP_gps_plus_lunar_data_piggyback_unwrapped.csv')):  
    #If calling this function repeatedly, reading in the csv separately and passing it will be more efficient. 
    #Returns the azimuth and altitude of the sun in radians at the location of the gondola.

    if timestamp > 1532000000:
        df = pd.read_csv('/data/mounted_filesystems/nas2/resources/SIP_gps_plus_lunar_data_piggyback_unwrapped.csv')
    else:
        print('Still need to generate csv file of lunar data for 2018 flight.')
        return
    i0 = np.where(df.epoch < timestamp)[0][-1]
    i1 = np.where(df.epoch > timestamp)[0][0]
    if (df.moon_az[i0] > 330 and df.moon_az[i1] < 30):
        az = df.moon_az[i0] + (timestamp - df.epoch[i0])/(df.epoch[i1] - df.epoch[i0])*(df.moon_az[i1] + 360 - df.moon_az[i0])
    else:
        az = df.moon_az[i0] + (timestamp - df.epoch[i0])/(df.epoch[i1] - df.epoch[i0])*(df.moon_az[i1] - df.moon_az[i0])
    az = az % 360
    
    alt = df.moon_alt[i0] + (timestamp - df.epoch[i0])/(df.epoch[i1] - df.epoch[i0])*(df.moon_alt[i1] - df.moon_alt[i0])
    
    return np.radians(az), np.radians(alt)

def get_moon_az_alt_array(timestamps):
    if timestamps[0] < 1532000000:
        print('Still need to generate csv file of lunar data for 2018 flight.')
        return
        #df = pd.read_csv('/data/mounted_filesystems/nas2/resources/SIP_gps_plus_solar_data_unwrapped.csv')
    else:
        df = pd.read_csv('/data/mounted_filesystems/nas2/resources/SIP_gps_plus_lunar_data_piggyback_unwrapped.csv')
    f_moon_az = inter.interp1d(df.epoch, df.moon_az)
    f_moon_alt = inter.interp1d(df.epoch, df.moon_alt)
    
    return np.radians(f_moon_az(timestamps)), np.radians(f_moon_alt(timestamps))


import os

import numpy as np
import scipy.stats
from scipy.interpolate import interp1d 

from skywinder_analysis.lib.tools import blosc_file, periodic_image_finder
from skywinder_analysis.lib.image_processing import flat_field, dark_images, sky_brightness_model

def make_composite_reflections_image(camera_number, start, end, interval):
    fns = periodic_image_finder.find_periodic_images_full_fn(camera_number, start, end, interval)
    imd = np.zeros((3232, 4864))
    for fn in fns:
        img = flat_field.get_foreground_subtracted_image(camera_number, fn)
        imd += 20000.0*img/float(len(fns))
    imd -= 2*np.min(imd)
    return imd 




from default_settings import settings

settings.camera_numbers = [1,2,3,4,5,6,7]
#settings.camera_numbers = [99]
#settings.camera_numbers = [4]
settings.bin_size = 4
settings.new_flat_field_window = 600
#settings.start = 1576540800  + (10*24*60*60)# 2019 12 17
settings.start = 1531353600  + (18*60*60)# 2018-07-12_0000
settings.stop = settings.start + (3*60*60)
settings.section_size = (10*60)
settings.interval = 2
settings.piggyback = False



import numpy as np
import sys
import csv

sys.path.append('/home/bjorn/astrometry.net-0.73/')
# This needs to be a custom setting. Refactor in future.

import tempfile
import os
import time
import subprocess
from skywinder_analysis.lib.pointing.fits import streaming_text_table

SOLVE_FIELD = 'solve-field'
WCSINFO = 'wcsinfo'
WCS_XY2RD = 'wcs-xy2rd'


def get_wcs_info(wcs_file):
    command = ' '.join([WCSINFO, wcs_file])
    wcs_info_string = subprocess.check_output(command, shell=True)
    result = {}
    for line in wcs_info_string.splitlines():
        k, v = line.split()
        try:
            v = float(v)
        except ValueError:
            pass
        result[k] = v
    return result


def get_wcs_xy2rd(wcs_file, xcoord, ycoord):
    command = ' '.join([WCS_XY2RD, '-x', str(xcoord), '-y', str(ycoord), '-w', wcs_file])
    wcs_xy2rd_string = subprocess.check_output(command, shell=True)
    return wcs_xy2rd_string


def get_mesh(xcoords, ycoords, work_dir=None, time_limit=10, verbose=False):
    # Get pointing for an image from list of xcoords and ycoords of stars in image.
    # Returns ra/dec for a mesh of points within the solved image.
    if work_dir is None:
        work_dir = tempfile.mkdtemp(suffix='pyadn_work')
    if verbose:
        print("working in %s" % work_dir)
        print("writing xy text file")
    text_xy = os.path.join(work_dir, 'xy.txt')
    with open(text_xy, 'w') as fh:
        for (x, y) in zip(xcoords, ycoords):
            fh.write('%f,%f\n' % (x, y))
    if verbose:
        print("creating fits file")
    stt = streaming_text_table(text_xy, split=",",
                               skiplines=0, headerline="x,y",
                               coltypes=[np.float64, np.float64], floatvalmap={'': np.nan},
                               intvalmap=dict(null=-1))
    fits_xy = os.path.join(work_dir, 'xy.fits')
    stt.write_to(fits_xy)

    command = [SOLVE_FIELD,
               fits_xy,
               '-l', str(time_limit),  # give up after this many seconds
               '-w', '3232',
               '-e', '4864',
               '--no-plot']
    command = ' '.join(command)

    tic = time.time()
    try:
        solve_result = subprocess.check_output(command, stderr=subprocess.STDOUT, shell=True)
    except subprocess.CalledProcessError as e:
        solve_result = e.output
    elapsed = time.time() - tic
    if verbose:
        print("solved in %.3f" % elapsed)
        print("Solve command: %s" % (' '.join(command)))
        print("Solve result:\n" + solve_result)
    with open(os.path.join(work_dir, 'solve.log'), 'wb') as fh:
        fh.write(solve_result)

    corners = []

    for x in [0, 3232 / 4, 3232 / 2, 3 * 3232 / 4, 3232]:
        for y in [0, 4864 / 4, 4864 / 2, 3 * 4864 / 4, 4864]:
            # Used for full PMC-Turbo images (3232, 4864)
            print(x, y)
            corner = get_wcs_xy2rd(os.path.join(work_dir, 'xy.wcs'), x, y)
            corners.append(corner.decode())

    corner_coords = []

    for corner in corners:
        corner = str(corner)
        text = corner.split('(')[-1]
        values = text.strip(')\n').split(', ')
        print(values)
        ra = float(values[0])
        dec = float(values[1])
        corner_coords.append((ra, dec))

    return corner_coords


def get_corner_and_blob_coords(xcoords, ycoords, blobs_csv_filename, work_dir=None, time_limit=10, verbose=False):
    # Get pointing for an image from list of xcoords and ycoords of stars in image.
    # Returns ra/dec for four corners of the image within the solved image,
    # as well as the ra/dec for each of the coordinates given in the blobs_csv_filename.

    # This can be simplified as it already takes a list of blob coords. They aren't necessarily the coords that
    # one wants, however.
    if work_dir is None:
        work_dir = tempfile.mkdtemp(suffix='pyadn_work')
    if verbose:
        print("working in %s" % work_dir)
        print("writing xy text file")
    text_xy = os.path.join(work_dir, 'xy.txt')
    with open(text_xy, 'w') as fh:
        for (x, y) in zip(xcoords, ycoords):
            fh.write('%f,%f\n' % (x, y))
    if verbose:
        print("creating fits file")
    stt = streaming_text_table(text_xy, split=",",
                               skiplines=0, headerline="x,y",
                               coltypes=[np.float64, np.float64], floatvalmap={'': np.nan},
                               intvalmap=dict(null=-1))
    fits_xy = os.path.join(work_dir, 'xy.fits')
    stt.write_to(fits_xy)

    command = [SOLVE_FIELD,
               fits_xy,
               '-l', str(time_limit),  # give up after this many seconds
               '-w', '3232',
               '-e', '4864',
               '--no-plot']
    command = ' '.join(command)

    tic = time.time()
    try:
        solve_result = subprocess.check_output(command, stderr=subprocess.STDOUT, shell=True)
    except subprocess.CalledProcessError as e:
        solve_result = e.output
    elapsed = time.time() - tic
    if verbose:
        print("solved in %.3f" % elapsed)
        print("Solve command: %s" % (' '.join(command)))
        print("Solve result:\n" + solve_result)
    with open(os.path.join(work_dir, 'solve.log'), 'w') as fh:
        fh.write(solve_result)

    corners = []

    for x in [0, 3232]:
        for y in [0, 4864]:
            print(x, y)
            corner = get_wcs_xy2rd(os.path.join(work_dir, 'xy.wcs'), x, y)
            corners.append(corner)

    corner_coords = []

    for corner in corners:
        text = corner.split('(')[-1]
        values = text.strip(')\n').split(', ')
        print(values)
        ra = float(values[0])
        dec = float(values[1])
        corner_coords.append((ra, dec))

    blobs_ = []
    blobs_ = []
    with open(blobs_csv_filename, 'rb') as csvfile:
        csvreader = csv.reader(csvfile)
        for row in csvreader:
            blobs_.append((int(row[0]), int(row[1])))

    blob_locations = []
    for blob_ in blobs_:
        x, y = blob_
        blob_location = get_wcs_xy2rd(os.path.join(work_dir, 'xy.wcs'), x, y)
        blob_locations.append(blob_location)

    blob_coords = []
    for blob_location in blob_locations:
        text = blob_location.split('(')[-1]
        values = text.strip(')\n').split(', ')
        print(values)
        ra = float(values[0])
        dec = float(values[1])
        blob_coords.append((ra, dec))

    return corner_coords, blob_coords



import os
import sys
import time

import matplotlib.animation as animation
import matplotlib.cm as cm
import matplotlib.pyplot as plt
import matplotlib.patches as patches
import numpy as np
import scipy.stats
from skywinder_analysis.lib.tools import blosc_file
from skywinder_analysis.lib.image_processing import binning, flat_field
from skywinder_analysis.lib.power_spectrum import power_spectrum_tools
from skywinder_analysis.lib.skywinder_analysis_app import skywinder_analysis_app


class FeatureAndContextMovieMakerApp(skywinder_analysis_app.App):
    def ani_frame_from_raw(self):
        self.create_output()
        self.logger.info('Starting animation of frames')
        imd = np.zeros(self.settings.raw_image_size)
        if self.settings.rotate:
            imd = np.rot90(imd, k=1)
        frames = len(self.settings.filenames)
        self.logger.info('%d filenames passed' % frames)
        fig, axs = plt.subplots(1, 2)
        for ax in axs:
            ax.get_xaxis().set_visible(False)
            ax.get_yaxis().set_visible(False)
        fig.subplots_adjust(left=0, bottom=0, right=1, top=1, wspace=None, hspace=None)
        feature = imd[self.settings.section_y:self.settings.section_y + self.settings.section_ywidth,
                  self.settings.section_x:self.settings.section_x + self.settings.section_xwidth]
        img = binning.bucket(imd, self.settings.bin_pixels)
        rect = patches.Rectangle((self.settings.section_x // self.settings.bin_pixels[0],
                                  self.settings.section_y // self.settings.bin_pixels[0]),
                                  self.settings.section_xwidth // self.settings.bin_pixels[0],
                                  self.settings.section_ywidth // self.settings.bin_pixels[0],
                                  linewidth=4,
                                  edgecolor='c',
                                  facecolor='none')

        feature = binning.bucket(feature, self.settings.feature_bin_pixels)
        if self.settings.rotate:
            img = np.rot90(img, k=1)
        axs[0].add_patch(rect)
        im0 = axs[0].imshow(img, cmap=cm.inferno, interpolation='nearest')
        im1 = axs[1].imshow(feature, cmap=cm.inferno, interpolation='nearest')

        vsize = 1 * img.shape[0] / self.settings.dpi
        hsize = 2.1 * img.shape[1] / self.settings.dpi
        fig.set_size_inches([hsize, vsize])
        xfig, yfig = fig.get_dpi() * fig.get_size_inches()
        fontsize = yfig / 32
        txt = axs[0].text(0.01 * xfig, 0.05 * yfig, 'time:', ha='left', va='top', color='black', fontsize=fontsize)
        start_at = time.time()
        if self.settings.saved_flat_field_path:
            self.logger.info('Loading saved flat field %s' % self.settings.saved_flat_field_path)
            flat_field_image = np.load(self.settings.saved_flat_field_path)
            use_flat_field_image = True
        elif self.settings.flat_field_filenames:
            self.logger.info('Generating flat field from %d files' % len(self.settings.flat_field_filenames))
            flat_field_image = flat_field.generate_flat_field_image_from_filenames(self.settings.flat_field_filenames)
            use_flat_field_image = True
            # Generate flat_field_image from filenames
        else:
            self.logger.info('No flat field used')
            use_flat_field_image = False

        def update_img(n):
            elapsed = time.time() - start_at
            if n:
                time_per_frame = elapsed / n
            else:
                time_per_frame = 1
            sys.stdout.flush()
            try:
                img, _ = blosc_file.load_blosc_image(self.settings.filenames[n])
                if use_flat_field_image:
                    img = flat_field.apply_flat_field(img, flat_field_image)
            except Exception as e:
                self.logger.info(e)
                return
            self.logger.info('X: %d, X-width: %d, Y: %d, Y-width: %d' % (self.settings.section_x, self.settings.section_xwidth, self.settings.section_y, self.settings.section_ywidth))
            feature = img[self.settings.section_y:self.settings.section_y + self.settings.section_ywidth,
                          self.settings.section_x:self.settings.section_x + self.settings.section_xwidth]
            self.logger.info('Shape of feature is %d by %d' % (feature.shape[0], feature.shape[1]))
            img = binning.bucket(img, self.settings.bin_pixels)
            feature = binning.bucket(feature, self.settings.feature_bin_pixels)
            rect.set_xy([self.settings.section_x // self.settings.bin_pixels[0], self.settings.section_y // self.settings.bin_pixels[0]])
            if self.settings.rotate:
                img = np.rot90(img, k=1)
            im0.set_data(img)
            vmin = scipy.stats.scoreatpercentile(img, self.settings.min_percentile)
            vmax = scipy.stats.scoreatpercentile(img, self.settings.max_percentile)
            self.logger.info('%.1f percentile is %.1f, %.1f percentile is %.1f' % (self.settings.min_percentile, vmin, self.settings.max_percentile, vmax))
            im0.set_clim(vmin, vmax)

            try:
                time_string = self.settings.filenames[n].split('/')[-1].split('_')[1]
                # Switch from Eastern time to UT
                hour = int(time_string[0:2])
                hour += 4
                hour = hour % 24

                time_string = ('Camera %d ' % self.settings.camera_number) + ('%d' % hour) \
                              + ':' + time_string[2:4] + ':' + time_string[4:]
                txt.set_text(time_string)
            except Exception:
                pass
            self.logger.info("\r%d of %d %.1f minutes elapsed, %.1f minutes remaining" % (
                n, len(self.settings.filenames), elapsed / 60,
                (len(self.settings.filenames) - n) * time_per_frame / 60))

            im1.set_data(feature)
            im1.set_clim(vmin, vmax)

            self.settings.section_x = self.settings.section_x + self.settings.xshift
            self.settings.section_y = self.settings.section_y + self.settings.yshift
            return im0, im1, txt

        ani = animation.FuncAnimation(fig, update_img, frames, interval=50)

        ani_fn = os.path.join(self.out_path, self.settings.output_name)
        ani.save(ani_fn, writer=self.settings.writer, dpi=self.settings.dpi)
        return ani


if __name__ == "__main__":
    app = FeatureAndContextMovieMakerApp()
    app.ani_frame_from_raw()
    app.end()



import logging

import blosc
import numpy as np

from skywinder_analysis.lib.tools import dtypes

logger = logging.getLogger(__name__)

# We need to ensure blosc uses just 1 thread so that it is always compatible with multiprocessing. This is true as of
#  blosc 1.4.4, but may improve in the future.
original_nthreads = blosc.set_nthreads(1)
logger.debug("Set blosc to use 1 thread, originally was using %d" % original_nthreads)


def load_blosc_file(filename):
    logger.debug("Reading blosc file from %s" % filename)
    with open(filename, 'rb') as fh:
        data = blosc.decompress(fh.read())
    return data

def load_blosc_image(filename):
    data = load_blosc_file(filename)
    image = np.frombuffer(data[:-dtypes.chunk_num_bytes], dtype='uint16')
    image.shape = dtypes.image_dimensions
    chunk_data = np.frombuffer(data[-dtypes.chunk_num_bytes:], dtype=dtypes.chunk_dtype)
    return image, chunk_data

def load_blosc_processed_image(filename):
    data = load_blosc_file(filename)
    image = np.frombuffer(data, dtype='uint16')
    image.shape = (808,1216)
    image2 = image - 2.*np.min(image)
    image2 /= 5.
    return image2

def write_image_blosc(filename, data):
    fh = open(filename, 'wb')
    compressed_data = blosc.compress(data, shuffle=blosc.BITSHUFFLE, cname='lz4')
    fh.write(compressed_data)
    fh.close()



from default_settings import settings
import numpy as np
import matplotlib.animation as animation
import datetime, calendar

import os
from pmc_analysis.lib.tools import periodic_image_finder
 
n = 1531402200 # 2018-07-12_1330
start = n + (4*60)
stop = start + (2*60)
interval = 10

settings.camera_numbers = [4,5,6,7]
settings.all_filename_lists = {4: [], 5:[], 6:[], 7:[]}
idxs = []
#for i in np.arange(55, 71, 0.5):
#    idxs.append(int(i*30))
for i in np.arange(55, 71, 1):
    idxs.append(int(i*30))
for cam_num in settings.camera_numbers:
    filenames = periodic_image_finder.find_periodic_images_full_fn(cam_num,
                                                                            start_timestamp=start,
                                                                            stop_timestamp=stop,
                                                                            interval=interval)

    #for idx in idxs:
    #    settings.all_filename_lists[cam_num].append(filenames[idx])
    settings.all_filename_lists[cam_num] = filenames



c4_filenames = []
c5_filenames = []
c6_filenames = []
c7_filenames = []
datetimes = [
#                datetime.datetime(2018, 7, 9, 13, 20),
#                datetime.datetime(2018, 7, 9, 14, 10),
#                datetime.datetime(2018, 7, 9, 17, 2),
#                datetime.datetime(2018, 7, 9, 17, 22),
#                datetime.datetime(2018, 7, 10, 2, 40),
#                datetime.datetime(2018, 7, 11, 3, 30),
#                datetime.datetime(2018, 7, 11, 5, 8),
#                datetime.datetime(2018, 7, 11, 5, 43),
#                datetime.datetime(2018, 7, 11, 8, 10),
#                datetime.datetime(2018, 7, 11, 8, 48),
#                datetime.datetime(2018, 7, 11, 9, 24),
#                datetime.datetime(2018, 7, 11, 23, 10),
                datetime.datetime(2018, 7, 12, 14, 8),
#                datetime.datetime(2018, 7, 13, 13, 32),
#                datetime.datetime(2018, 7, 13, 21, 55),
#                datetime.datetime(2018, 7, 14, 1, 14),
            ]
timestamps = []
for datetime in datetimes:
    timestamps.append(calendar.timegm(datetime.utctimetuple()))

for timestamp in timestamps:
    c4_filenames.append(periodic_image_finder.find_periodic_images_full_fn(4, timestamp, timestamp+2, 2)[0])
    c5_filenames.append(periodic_image_finder.find_periodic_images_full_fn(5, timestamp, timestamp+2, 2)[0])
    c6_filenames.append(periodic_image_finder.find_periodic_images_full_fn(6, timestamp, timestamp+2, 2)[0])
    c7_filenames.append(periodic_image_finder.find_periodic_images_full_fn(7, timestamp, timestamp+2, 2)[0])

settings.all_filename_lists[4] = c4_filenames
settings.all_filename_lists[5] = c5_filenames
settings.all_filename_lists[6] = c6_filenames
settings.all_filename_lists[7] = c7_filenames




settings.min_x = -100
settings.max_x = 100
settings.x_increment = 200

settings.min_y = 0
settings.max_y = 115
settings.y_increment = 100

settings.bin_factor = 8

#settings.min_x = -50
#settings.max_x = 0
#settings.x_increment = 1000
#
#settings.min_y = 25
#settings.max_y = 75
#settings.y_increment = 1000

settings.xvalues = np.linspace(settings.min_x, settings.max_x, num=settings.x_increment)
settings.yvalues = np.linspace(settings.min_y, settings.max_y, num=settings.y_increment)

settings.individual_frames = True

settings.grid = True
settings.grid_spacing = 5



from default_settings import settings
import numpy as np
import matplotlib.animation as animation
import matplotlib.cm as cm
import os
import glob
import datetime
from pmc_analysis.lib.tools import periodic_image_finder


#n = 1578136740 # 2020 01 04 0619
#start = n
#stop = start + (1*60) 
#interval = 10

fns = glob.glob('/home/bjorn/cips_piggyback_overlap_maps/overlaps/*')
unix_timestamps = []

for fn in fns:
    f = fn.split('/')[-1]
    y=int(f[0:4])
    m=int(f[5:7])
    d=int(f[8:10])
    hr=int(f[11:13])
    mn=int(f[14:16])
    dt = datetime.datetime(y,m,d,hr,mn)
    unix_ts = dt.replace(tzinfo=datetime.timezone.utc).timestamp()
    unix_timestamps.append(unix_ts)

settings.cam_nums = [99]
settings.all_filename_lists = {99: []}
#settings.camera_numbers = [4,5,6,7]
#settings.all_filename_lists = {4: [], 5:[], 6:[], 7:[]}
for cam_num in settings.cam_nums:
    #filenames = periodic_image_finder.find_periodic_images_full_fn(cam_num,
    #                                                                        start_timestamp=start,
    #                                                                        stop_timestamp=stop,
    #                                                                        interval=interval)
    filenames = []
    for unix_ts in unix_timestamps:
        start = unix_ts
        stop = start + (15*60)+1
        interval = 300
        filenames += periodic_image_finder.find_periodic_images_full_fn(99, start, stop, interval)
    settings.all_filename_lists[cam_num] = filenames

settings.min_x = -35
settings.max_x = 30
settings.x_increment = 400

settings.min_y = 30
settings.max_y = 95
settings.y_increment = 400

settings.bin_factor = 8

settings.xvalues = np.linspace(settings.min_x, settings.max_x, num=settings.x_increment)
settings.yvalues = np.linspace(settings.min_y, settings.max_y, num=settings.y_increment)
settings.individual_frames = True
settings.grid = True
settings.grid_spacing = 5
settings.color_scheme = cm.inferno

settings.timestamp_in_image = True

settings.piggyback = True

settings.centerline = 215.33 # Centerline for piggyback solution



from default_settings import settings
import numpy as np
import matplotlib.animation as animation
import matplotlib.cm as cm


import os
from pmc_analysis.lib.tools import periodic_image_finder
n = 1531402200 # 2018-07-12_1330
#n = 1531165800 # 2018-07-09_1250
#n = 1531155000 # 2018-07-09_1650
#start = n + (10*60)
#stop = start + (18*60)
start = n + (3*60) + 40
stop = start + 30
interval = 30
#settings.camera_numbers = [4,5,6,7]
#settings.all_filename_lists = {4: [], 5:[], 6:[], 7:[]}
settings.cam_nums = [4, 5]
settings.all_filename_lists = {4:[], 5:[]}
for cam_num in settings.cam_nums:
    filenames = periodic_image_finder.find_periodic_images_full_fn(cam_num,
                                                                            start_timestamp=start,
                                                                            stop_timestamp=stop,
                                                                            interval=interval)

    settings.all_filename_lists[cam_num] = filenames

settings.min_x = -70
settings.max_x = settings.min_x + 55
settings.x_increment = 5500 

settings.min_y = 30
settings.max_y = settings.min_y + 40
settings.y_increment = 4000

settings.bin_factor = 4

settings.xvalues = np.linspace(settings.min_x, settings.max_x, num=settings.x_increment)
settings.yvalues = np.linspace(settings.min_y, settings.max_y, num=settings.y_increment)
settings.individual_frames = True
settings.grid = True
settings.grid_spacing = 5
settings.piggyback = False

#settings.color_scheme = cm.Blues_r
settings.timestamp_in_image = False
